import { __decorate } from "tslib";
import { css, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { Collapse } from '../collapse/index.js';
import { VERSION } from '../version.js';
/**
 * Finds closest accordion parent of element.
 * @param element - potential child of accordion
 * @returns found accordion parent or null, if not found
 */
const getClosestAccordion = (element) => {
    return element.closest('ef-accordion');
};
/**
 * Checks if specified element is a direct child of current accordion.
 * @param element - child that checked
 * @param accordion - parent accordion
 * @returns is current accordion has child accordion that wraps checked element
 */
const isDirectAccordionChild = (element, accordion) => {
    return element instanceof Collapse && getClosestAccordion(element) === accordion;
};
/**
 * Used to display a group of `Collapse` control.
 * Only one item will be able to expand by default but you can customize its behavior.
 *
 * @slot header-left - Slot to add custom contents to the left side of header
 * @slot header-right - Slot to add custom contents to the right side of header
 *
 */
let Accordion = class Accordion extends Collapse {
    constructor() {
        super(...arguments);
        /**
         * Allow multiple sections expand at the same time
         */
        this.autoCollapseDisabled = false;
        /**
         * Add spacing to content section in all collapse items
         */
        this.spacing = false;
        /**
         * handle bubbled clicks from items
         * @param event the click event object
         * @return void
         */
        this.handleClick = (event) => {
            if (!this.autoCollapseDisabled && isDirectAccordionChild(event.target, this)) {
                this.processChildrenOnClick(event.target);
                event.stopPropagation();
            }
        };
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @return CSS template
     */
    static get styles() {
        return css `
      :host {
        display: block;
      }
    `;
    }
    /**
     * Called once after the component is first rendered
     * @param changedProperties map of changed properties with old values
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.addEventListener('expanded-changed', this.handleClick, true);
    }
    /**
     * get a list of items
     * @returns array of accordion items
     */
    getChildItems() {
        return [...this.querySelectorAll('ef-collapse')].filter((el) => isDirectAccordionChild(el, this));
    }
    /**
     * collapse non selected child items on click
     * @param target currently selected item
     * @return void
     */
    processChildrenOnClick(target) {
        const children = this.getChildItems();
        for (let i = 0, len = children.length; i < len; ++i) {
            if (children[i] !== target) {
                children[i].expanded = false;
            }
        }
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return {TemplateResult}  Render template
     */
    render() {
        return html ` <slot></slot> `;
    }
};
__decorate([
    property({ type: Boolean, attribute: 'auto-collapse-disabled' })
], Accordion.prototype, "autoCollapseDisabled", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Accordion.prototype, "spacing", void 0);
Accordion = __decorate([
    customElement('ef-accordion')
], Accordion);
export { Accordion };
