import { __decorate } from "tslib";
import { BasicElement, css, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { query } from '@refinitiv-ui/core/decorators/query.js';
import { state } from '@refinitiv-ui/core/decorators/state.js';
import '@refinitiv-ui/phrasebook/locale/en/card.js';
import { translate } from '@refinitiv-ui/translate';
import { isSlotEmpty } from '@refinitiv-ui/utils/is-slot-empty.js';
import '../button/index.js';
import '../label/index.js';
import '../overlay-menu/index.js';
import { VERSION } from '../version.js';
/**
 * A card frame component.
 *
 * @fires item-trigger - Fired when the user selects card menu.
 *
 * @slot header - Adds slotted content into the header of the card.
 * @slot footer - Adds slotted content into the footer of the card.
 */
let Card = class Card extends BasicElement {
    constructor() {
        super(...arguments);
        this._config = {};
        /**
         * Set text on the header
         */
        this.header = '';
        /**
         * Set text on the footer
         */
        this.footer = '';
        /**
         * Menu open state
         */
        this.menuOpened = false;
        /**
         * True if header has slotted content
         */
        this.headerHasContent = false;
        /**
         * True if footer has slotted content
         */
        this.footerHasContent = false;
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @returns CSS template
     */
    static get styles() {
        return css `
      :host {
        display: flex;
        flex-flow: column nowrap;
      }

      [part~='header'] {
        display: flex;
      }

      [part~='header-body'] {
        flex: 1;
        min-width: 0px;
      }

      [part~='body'] {
        overflow: auto;
        flex: 1;
      }

      [part~='header'],
      [part~='footer'] {
        flex: 0 0 auto;
      }

      [part~='footer']:not([part~='has-content']),
      [part~='header']:not([part~='has-content']) {
        display: none;
      }
    `;
    }
    /**
     * Set card configurations
     * @type {CardConfig}
     * @default {}
     */
    get config() {
        return this._config;
    }
    set config(config) {
        const data = config?.menu?.data;
        if (data !== this.menuData) {
            this.menuData = data;
        }
        this._config = config;
    }
    /**
     * Open menu
     * @returns {void}
     */
    openMenu() {
        if (this.menuElement && !(this.menuElement.fullyOpened || this.menuElement.transitioning)) {
            this.menuElement.opened = true;
            this.menuOpened = true;
        }
    }
    /**
     * Close menu
     * @param event ItemTriggerEvent
     * @returns {void}
     */
    closeMenu(event) {
        if (this.menuElement?.opened && event.detail.value) {
            this.menuElement.opened = false;
            this.menuOpened = false;
        }
    }
    /**
     * Run on header slot slotchange
     * @param event Footer slotchange event
     * @returns {void}
     */
    onHeaderSlotChange(event) {
        this.headerHasContent = isSlotEmpty(event.target);
    }
    /**
     * Run on footer slot slotchange
     * @param event Header slotchange event
     * @returns {void}
     */
    onFooterSlotChange(event) {
        this.footerHasContent = isSlotEmpty(event.target);
    }
    /**
     * Run on overlay menu open changed
     * @param event overlay menu opened changed event
     * @returns {void}
     */
    onMenuOpenChanged(event) {
        this.menuOpened = event.detail.value;
    }
    /**
     * True if card has header
     */
    get withHeader() {
        return this.headerHasContent || !!this.header || !!this.menuData;
    }
    /**
     * True if card has footer
     */
    get withFooter() {
        return this.footerHasContent || !!this.footer;
    }
    /**
     * Called after render life-cycle finished
     * @param changedProperties Properties which have changed
     * @return {void}
     */
    updated(changedProperties) {
        super.updated(changedProperties);
        if (changedProperties.has('menuData') && this.menuElement) {
            this.menuElement.positionTarget = this.openMenuElement;
        }
    }
    /**
     * Called after the component is first rendered
     * @param changedProperties Properties which have changed
     * @return {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.addEventListener('item-trigger', (event) => this.closeMenu(event)); // Here to cover nested menus
    }
    /**
     * Template of menu
     * @return menu template
     */
    get menuTemplate() {
        return html `${this.menuData
            ? html ` <ef-button
            part="menu-button"
            icon="more-vertical"
            transparent
            aria-label=${this.t('OPEN_MENU')}
            aria-haspopup="true"
            aria-controls="menu-popup"
            aria-expanded=${this.menuOpened}
            @tap=${this.openMenu}
          ></ef-button>
          <ef-overlay-menu
            id="menu-popup"
            part="menu-popup"
            .data=${this.menuData}
            position="bottom-end"
            @opened-changed=${this.onMenuOpenChanged}
          ></ef-overlay-menu>`
            : undefined} `;
    }
    /**
     * Template of header
     * @return header template
     */
    get headerTemplate() {
        return html `
      <div part="header${this.withHeader ? ' has-content' : ''}">
        <div part="header-body">
          <slot name="header" @slotchange="${this.onHeaderSlotChange}"></slot>
          ${!this.headerHasContent && this.header
            ? html `<ef-label line-clamp="3" part="header-text">${this.header}</ef-label>`
            : null}
        </div>
        ${this.menuTemplate}
      </div>
    `;
    }
    /**
     * Template of footer
     * @return footer template
     */
    get footerTemplate() {
        return html `
      <div part="footer${this.withFooter ? ' has-content' : ''}">
        <div part="footer-body">
          <slot name="footer" @slotchange="${this.onFooterSlotChange}"></slot>
          ${!this.footerHasContent && this.footer
            ? html `<ef-label line-clamp="3">${this.footer}</ef-label>`
            : undefined}
        </div>
      </div>
    `;
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return html `
      ${this.headerTemplate}
      <div part="body"><slot></slot></div>
      ${this.footerTemplate}
    `;
    }
};
__decorate([
    property({ type: String })
], Card.prototype, "header", void 0);
__decorate([
    property({ type: String })
], Card.prototype, "footer", void 0);
__decorate([
    property({ type: Object, attribute: false })
], Card.prototype, "config", null);
__decorate([
    query('[part=menu-popup]')
], Card.prototype, "menuElement", void 0);
__decorate([
    query('[part=menu-button]')
], Card.prototype, "openMenuElement", void 0);
__decorate([
    translate()
], Card.prototype, "t", void 0);
__decorate([
    state()
], Card.prototype, "menuData", void 0);
__decorate([
    state()
], Card.prototype, "menuOpened", void 0);
__decorate([
    state()
], Card.prototype, "headerHasContent", void 0);
__decorate([
    state()
], Card.prototype, "footerHasContent", void 0);
Card = __decorate([
    customElement('ef-card')
], Card);
export { Card };
