import { __decorate } from "tslib";
import { ControlElement, css, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { query } from '@refinitiv-ui/core/decorators/query.js';
import '../icon/index.js';
import { registerOverflowTooltip } from '../tooltip/index.js';
import { VERSION } from '../version.js';
/**
 * Form control for selecting one or several options
 * @fires checked-changed - Fired when the user checks or unchecks the checkbox. The event is not fired if the `checked` property is changed programmatically.
 *
 * @attr {boolean} readonly - Set readonly state
 * @prop {boolean} [readonly=false] - Set readonly state
 *
 * @attr {boolean} disabled - Set disabled state
 * @prop {boolean} [disabled=false] - Set disabled state
 */
let Checkbox = class Checkbox extends ControlElement {
    constructor() {
        super(...arguments);
        this.defaultRole = 'checkbox';
        /**
         * Value of checkbox
         */
        this.checked = false;
        /**
         * Set indeterminate state
         */
        this.indeterminate = false;
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @return CSS template
     */
    static get styles() {
        return css `
      :host {
        display: inline-block;
      }
      [part='check'] {
        visibility: hidden;
      }
      :host([checked]) [part='check'],
      :host([indeterminate]) [part='check'] {
        visibility: inherit;
      }
      [part='label'],
      [part='container'] {
        display: inline-block;
        vertical-align: middle;
      }
      [part='label'] {
        white-space: nowrap;
        text-overflow: ellipsis;
        overflow: hidden;
      }
      :host(:empty) [part='label'],
      [part='label']:empty {
        display: none;
      }
    `;
    }
    /**
     * Called before update() to compute values needed during the update.
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    willUpdate(changedProperties) {
        super.willUpdate(changedProperties);
        if (changedProperties.has('checked')) {
            if (this.checked) {
                this.indeterminate = false;
            }
            this.setAttribute('aria-checked', String(this.checked));
        }
        if (changedProperties.has('indeterminate')) {
            if (this.indeterminate) {
                this.checked = false;
            }
            this.setAttribute('aria-checked', this.indeterminate ? 'mixed' : String(this.checked));
        }
    }
    /**
     * Called once after the component is first rendered
     * @param changedProperties map of changed properties with old values
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.addEventListener('tap', this.onTap);
        this.addEventListener('keydown', this.onKeyDown);
        registerOverflowTooltip(this.labelEl, () => this.textContent);
    }
    /**
     * Run when checkbox is tapped
     * @param event Tap event
     * @returns {void}
     */
    onTap(event) {
        if (this.disabled || this.readonly || event.defaultPrevented) {
            return;
        }
        this.handleChangeChecked();
    }
    /**
     * Handles key down event
     * @param event Key down event object
     * @returns {void}
     */
    onKeyDown(event) {
        if (this.disabled || this.readonly || event.defaultPrevented) {
            return;
        }
        switch (event.key) {
            case ' ':
                this.handleChangeChecked();
                break;
            default:
                return;
        }
        event.preventDefault();
    }
    /**
     * Change checked state and fire
     * checked-changed event
     * @return {void}
     */
    handleChangeChecked() {
        this.checked = !this.checked;
        this.notifyPropertyChange('checked', this.checked);
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return {TemplateResult}  Render template
     */
    render() {
        return html `
      <div part="container">
        <div part="check">${!this.indeterminate ? html `<ef-icon icon="tick"></ef-icon>` : null}</div>
      </div>
      <div part="label"><slot></slot></div>
    `;
    }
};
__decorate([
    property({ type: Boolean, reflect: true })
], Checkbox.prototype, "checked", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Checkbox.prototype, "indeterminate", void 0);
__decorate([
    query('[part=label]', true)
], Checkbox.prototype, "labelEl", void 0);
Checkbox = __decorate([
    customElement('ef-checkbox')
], Checkbox);
export { Checkbox };
