import { __decorate } from "tslib";
import { BasicElement, css, html, nothing } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { state } from '@refinitiv-ui/core/decorators/state.js';
import { createRef, ref } from '@refinitiv-ui/core/directives/ref.js';
import '../header/index.js';
import '../icon/index.js';
import '../panel/index.js';
import { VERSION } from '../version.js';
/**
 * Allows users to hide non-critical information
 * or areas of the screen, maximizing the amount of real estate
 * for their primary displays.
 *
 * @fires expanded-changed - Fired when the user expands or collapses the control. The event is not triggered if `expanded` property is changed programmatically.
 *
 * @slot header-left - Slot to add custom contents to the left side of header
 * @slot header-right - Slot to add custom contents to the right side of header
 */
let Collapse = class Collapse extends BasicElement {
    constructor() {
        super(...arguments);
        /**
         * Set text on the header
         */
        this.header = '';
        /**
         * Use level styling from theme
         */
        this.level = '3';
        /**
         * Set to expand the item
         */
        this.expanded = false;
        /**
         * Set to apply padding from theme to content section
         */
        this.spacing = false;
        /**
         * An ef-panel wrapper
         */
        this.panelHolderRef = createRef();
        /**
         * A panel used to display content
         */
        this.panelRef = createRef();
        /**
         * Used to control aria-level for heading
         */
        this.headingLevel = null;
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @return CSS template
     */
    static get styles() {
        return css `
      :host {
        display: block;
      }
      :host(:not([expanded])) [part~='content'] {
        visibility: hidden;
      }
      [part~='content'] {
        overflow: hidden;
        box-sizing: border-box;
      }
      [no-animation] {
        animation: none !important;
      }
    `;
    }
    /**
     * Observes attribute change for `attributeChangedCallback`
     */
    static get observedAttributes() {
        const observed = super.observedAttributes;
        return ['aria-level'].concat(observed);
    }
    /**
     * Run when observed attributes change value
     * @param name attribute name
     * @param oldValue old attribute value
     * @param newValue new attribute value
     * @returns {void}
     */
    attributeChangedCallback(name, oldValue, newValue) {
        super.attributeChangedCallback(name, oldValue, newValue);
        if (name === 'aria-level') {
            this.headingLevel = newValue;
        }
    }
    /**
     * Called once after the component is first rendered
     * @param changedProperties map of changed properties with old values
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.panelHolderRef.value?.setAttribute('no-animation', '');
    }
    /**
     * Invoked whenever the element is updated
     * @param changedProperties map of changed properties with old values
     * @return {void}
     */
    updated(changedProperties) {
        super.updated(changedProperties);
        if (changedProperties.has('expanded')) {
            this.showHide();
        }
    }
    /**
     * Toggle the item
     * @returns {void}
     */
    toggle() {
        this.expanded = !this.expanded;
        const event = this.notifyPropertyChange('expanded', this.expanded, true);
        if (!event) {
            // revert expanded if event is cancelled
            this.expanded = !this.expanded;
        }
    }
    /**
     * Show or Hide the item depending on the expanded state
     * @returns {void}
     */
    showHide() {
        this.panelHolderRef.value?.removeAttribute('no-animation');
        this.setAnimationTargetHeight(this.getContentHeight());
    }
    /**
     * Set current content height at the target-height
     * @param height number or null value
     * @returns {void}
     */
    setAnimationTargetHeight(height) {
        this.updateVariable('--target-height', `${height}px`);
    }
    /**
     * Gets the height of the ef-panel element which contains the content
     * will pass height including optional spacing
     * @returns clientHeight of the panel so that the panel holder max-height can be set
     */
    getContentHeight() {
        const panelEl = this.panelRef.value;
        return panelEl ? panelEl.clientHeight : 0;
    }
    /**
     * Prevents expanding collapse when interactive element in slot is tapped
     * @param event Tap event
     * @returns {void}
     */
    handleSlotTap(event) {
        event.stopPropagation();
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return html `
      <ef-header
        part="header"
        role="heading"
        level=${this.level}
        aria-level=${this.headingLevel ?? nothing}
        @tap=${this.toggle}
      >
        <div
          id="header-label"
          part="header-label"
          role="button"
          tabindex="0"
          aria-expanded="${this.expanded}"
          aria-controls="content"
        >
          ${this.header}
        </div>
        <ef-icon icon="right" part="toggle" slot="left" aria-hidden="true"></ef-icon>
        <slot name="header-left" slot="left" @tap=${this.handleSlotTap}></slot>
        <slot name="header-right" slot="right" @tap=${this.handleSlotTap}></slot>
      </ef-header>
      <div
        ${ref(this.panelHolderRef)}
        id="content"
        part="content"
        role="region"
        aria-labelledby="header-label"
      >
        <ef-panel ${ref(this.panelRef)} part="content-data" ?spacing="${this.spacing}" transparent>
          <slot></slot>
        </ef-panel>
      </div>
    `;
    }
};
__decorate([
    property({ type: String })
], Collapse.prototype, "header", void 0);
__decorate([
    property({ type: String })
], Collapse.prototype, "level", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Collapse.prototype, "expanded", void 0);
__decorate([
    property({ type: Boolean })
], Collapse.prototype, "spacing", void 0);
__decorate([
    state()
], Collapse.prototype, "headingLevel", void 0);
Collapse = __decorate([
    customElement('ef-collapse')
], Collapse);
export { Collapse };
