import { __decorate } from "tslib";
import { BasicElement, svg } from '@refinitiv-ui/core';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { query } from '@refinitiv-ui/core/decorators/query.js';
import { isHex, rgb } from '@refinitiv-ui/utils/color.js';
import { VERSION } from '../../version.js';
/**
 * Element base class usually used
 * for creating palettes elements.
 */
export class Palettes extends BasicElement {
    constructor() {
        super(...arguments);
        /**
         * Color value in hex
         */
        this.value = '';
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * Create selector template
     * @return {SVGTemplateResult} selector template
     */
    get SelectorTemplate() {
        return svg `
        <polygon class="color-selector-shadow"></polygon>
        <polygon class="color-selector"></polygon>
      `;
    }
    /**
     * Hide selector element
     * @return {void}
     */
    hideSelector() {
        if (this.colorSelector && this.colorSelectorShadow) {
            this.colorSelector.style.display = 'none';
            this.colorSelectorShadow.style.display = 'none';
        }
    }
    /**
     * Show selector element on specific points
     * @return {void}
     * @param points points of colorSelector
     */
    showSelector(points) {
        if (this.colorSelector && this.colorSelectorShadow) {
            this.colorSelector.style.display = '';
            this.colorSelectorShadow.style.display = '';
            this.colorSelectorShadow.setAttribute('points', points);
            this.colorSelector.setAttribute('points', points);
        }
    }
    /**
     * Update color value when tab on color item
     * @param event mouse event
     * @return {void}
     */
    onTapItem(event) {
        const polygonElement = event.target;
        this.updateValue(polygonElement);
    }
    /**
     * Update color value when drag on color item
     * @param event mouse event
     * @return {void}
     */
    onMousemove(event) {
        const polygonElement = event.target;
        let mouseButton = event.buttons;
        if (mouseButton === undefined) {
            // buttons property is not supported in safari
            mouseButton = event.which;
        }
        if (mouseButton !== 1) {
            return;
        }
        // only update when user click and drag on color item
        this.updateValue(polygonElement);
    }
    /**
     * Update color value when drag on color item in mobile device
     * @param event touch event
     * @return {void}
     */
    onTouchmove(event) {
        const touchOffsets = event.changedTouches[0];
        const realTarget = this.shadowRoot?.elementFromPoint(touchOffsets.clientX, touchOffsets.clientY);
        this.updateValue(realTarget);
    }
    /**
     * Update color value and fired value-changed event
     * @param element target element to get value
     * @return {void}
     */
    updateValue(element) {
        const color = element.getAttribute('fill');
        if (color) {
            this.value = color;
            this.notifyPropertyChange('value', color);
        }
    }
    /**
     * Expand short format hex into long format
     * For instance, "#fff" becomes "#ffffff"
     * @param hex hex to expand
     * @returns expanded hex value
     */
    expandHex(hex) {
        return isHex(hex) ? rgb(this.value).formatHex() : '';
    }
}
__decorate([
    query('.color-selector')
], Palettes.prototype, "colorSelector", void 0);
__decorate([
    query('.color-selector-shadow')
], Palettes.prototype, "colorSelectorShadow", void 0);
__decorate([
    property({ type: String })
], Palettes.prototype, "value", void 0);
