import { isHex, rgb } from '@refinitiv-ui/utils/color.js';
const rgbNumberToString = (value) => (isNaN(value) ? '' : `${value}`); // replace NaN with empty string
/**
 * A helper model in order to keep and update values between RGB and HEX
 */
class ValueModel {
    /**
     * Create the new value model
     * @param value A valid hex value (e.g. "#ffffff") or an empty string
     */
    constructor(value = '') {
        this.initialValue = value;
        const { r, g, b } = rgb(value);
        this._red = rgbNumberToString(r);
        this._green = rgbNumberToString(g);
        this._blue = rgbNumberToString(b);
        this._hex = value.slice(1);
    }
    getHexValue() {
        if (!this.isValidRGB() || (this.red === '' && this.green === '' && this.blue === '')) {
            return '';
        }
        const hex = rgb(Number(this.red), Number(this.green), Number(this.blue)).formatHex();
        return hex ? hex.slice(1) : '';
    }
    isValidRGBValue(value) {
        return value === '' || (Number(value) >= 0 && Number(value) <= 255);
    }
    isValidHexValue(value) {
        return value === '' || isHex(`#${value}`);
    }
    isValidRGB() {
        return (this.isValidRGBValue(this.red) && this.isValidRGBValue(this.green) && this.isValidRGBValue(this.blue));
    }
    isValidHex() {
        return this.isValidHexValue(this.hex);
    }
    /**
     * Check if new value is different to initial
     * ignoring hex length
     * @returns true if different
     */
    hasChanged() {
        // Avoid the same hex color format of empty string and black color
        return ((this.initialValue !== this.value && (!!this.initialValue && !!this.value) === false) ||
            rgb(this.initialValue).formatHex() !== rgb(this.value).formatHex());
    }
    /**
     * Check if RGB and Hex values are valid
     * @returns true if both are valid
     */
    isValid() {
        return this.isValidRGB() && this.isValidHex();
    }
    /**
     * Set red color from RGB pallette.
     * Setting this overrides hex
     * @param red Red color, the string number from 0 - 255
     */
    set red(red) {
        this._red = red;
        this._green = this._green || '0';
        this._blue = this._blue || '0';
        this._hex = this.getHexValue();
    }
    /**
     * Get red color from RGB pallette
     */
    get red() {
        return this._red;
    }
    /**
     * Set green color from RGB pallette.
     * Setting this overrides hex
     * @param green Green color, the string number from 0 - 255
     */
    set green(green) {
        this._green = green;
        this._red = this._red || '0';
        this._blue = this._blue || '0';
        this._hex = this.getHexValue();
    }
    /**
     * Get green color from RGB pallette
     */
    get green() {
        return this._green;
    }
    /**
     * Set blue color from RGB pallette.
     * Setting this overrides hex
     * @param blue Blue color, the string number from 0 - 255
     */
    set blue(blue) {
        this._blue = blue;
        this._red = this._red || '0';
        this._green = this._green || '0';
        this._hex = this.getHexValue();
    }
    /**
     * Get blue color from RGB pallette
     */
    get blue() {
        return this._blue;
    }
    /**
     * Set hex value. Setting hex overrides RGB
     * @param hex Hex value, e.g. "ffffff" or "#ffffff"
     */
    set hex(hex) {
        if (!this.isValidHexValue(hex)) {
            this._red = '';
            this._green = '';
            this._blue = '';
        }
        else {
            const { r, g, b } = rgb(`#${hex}`);
            this._red = rgbNumberToString(r);
            this._green = rgbNumberToString(g);
            this._blue = rgbNumberToString(b);
        }
        this._hex = hex;
    }
    /**
     * Get the hex value (hex without #, e.g "ffffff").
     */
    get hex() {
        return this._hex;
    }
    /**
     * Get the value. Value is Hex with #, e.g. "#ffffff"
     */
    get value() {
        return this._hex ? `#${this._hex}` : '';
    }
}
export { ValueModel };
