import { __decorate } from "tslib";
import { BasicElement, css, svg } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { unsafeHTML } from '@refinitiv-ui/core/directives/unsafe-html.js';
import { Deferred } from '@refinitiv-ui/utils/loader.js';
import { VERSION } from '../version.js';
import { FlagLoader } from './utils/FlagLoader.js';
export { preload } from './utils/FlagLoader.js';
const EmptyTemplate = svg ``;
let Flag = class Flag extends BasicElement {
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @returns CSS template
     */
    static get styles() {
        return css `
      :host {
        display: inline-block;
        box-sizing: border-box;
        width: 1.33em;
        height: 1em;
      }
      svg {
        display: block;
        box-sizing: border-box;
        width: 100%;
        height: 100%;
      }
    `;
    }
    /**
     * Code of a known flag to render or URL of flag SVG file.
     * @example gb | https://cdn.io/flags/gb.svg
     * @default null
     */
    get flag() {
        return this._flag;
    }
    set flag(value) {
        const oldValue = this._flag;
        if (oldValue !== value) {
            this.deferFlagReady();
            this._flag = value;
            void this.setFlagSrc();
            this.requestUpdate('flag', oldValue);
        }
    }
    /**
     * The flag template to render
     */
    get template() {
        return this._template;
    }
    set template(value) {
        if (this._template !== value) {
            this._template = value;
            this.requestUpdate();
        }
        this.flagReady.resolve();
    }
    constructor() {
        super();
        this._flag = null;
        this._template = EmptyTemplate;
        this.flagReady = new Deferred();
        // `flagReady` resolves at this stage so that `updateComplete` would be resolvable
        // even in the case that `flag` attribute is missing.
        this.flagReady.resolve();
    }
    /**
     * Called after the component is first rendered
     * @param changedProperties Properties which have changed
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.setPrefix();
    }
    async getUpdateComplete() {
        const result = await super.getUpdateComplete();
        await this.flagReady.promise;
        return result;
    }
    /**
     * instantiate a new deferred promise for flag ready if it's not pending already
     * @returns {void}
     */
    deferFlagReady() {
        if (this.flagReady.isPending()) {
            return;
        }
        this.flagReady = new Deferred();
    }
    /**
     * Helper method, used to set the flag src.
     * @returns {void}
     */
    async setFlagSrc() {
        const value = this.flag ? await FlagLoader.getSrc(this.flag) : this.flag;
        if (value) {
            await this.loadAndRenderFlag(value);
        }
        else {
            this.clearFlag();
        }
    }
    /**
     * Tries to load an flag from the url provided
     * and the renders this into the flag template.
     * @param src Source location of the svg flag.
     * @returns {void}
     */
    async loadAndRenderFlag(src) {
        const svgBody = await FlagLoader.loadSVG(src);
        this.template = svgBody ? svg `${unsafeHTML(svgBody)}` : EmptyTemplate;
    }
    /**
     * Get and cache CDN prefix
     * This is a private URL which is set in the theme
     * and should not be configured again via the variable.
     * @returns {void}
     */
    setPrefix() {
        if (FlagLoader.isPrefixPending) {
            const CDNPrefix = this.getComputedVariable('--cdn-prefix').replace(/^('|")|('|")$/g, '');
            FlagLoader.setCdnPrefix(CDNPrefix);
        }
    }
    /**
     * Clears SVG body from the flag template
     * @returns {void}
     */
    clearFlag() {
        this.template = EmptyTemplate;
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return {TemplateResult} Render template
     */
    render() {
        return this.template;
    }
};
__decorate([
    property({ type: String, reflect: true })
], Flag.prototype, "flag", null);
Flag = __decorate([
    customElement('ef-flag')
], Flag);
export { Flag };
