import { __decorate } from "tslib";
import { consume } from '@lit-labs/context';
import { BasicElement, css, svg } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { unsafeSVG } from '@refinitiv-ui/core/directives/unsafe-svg.js';
import { Deferred, isBase64svg, isUrl } from '@refinitiv-ui/utils/loader.js';
import { efConfig } from '../configuration/index.js';
import { VERSION } from '../version.js';
import { IconLoader } from './utils/IconLoader.js';
import { SpriteLoader } from './utils/SpriteLoader.js';
const EmptyTemplate = svg ``;
/**
 * Cache for reusing SVG template results across multiple icons.
 * Reusing these templates increases performance dramatically when many icons are rendered.
 * As the cache key is an absolute URL, we can assume no clashes will occur.
 */
const iconTemplateCache = new Map();
let Icon = class Icon extends BasicElement {
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @return CSS template
     */
    static get styles() {
        return css `
      :host {
        display: inline-block;
        line-height: 1;
        width: 1em;
        height: 1em;
      }
      svg {
        width: 100%;
        height: 100%;
      }
    `;
    }
    /**
     * Name of a known icon to render or URL of SVG icon.
     * @example heart | https://cdn.io/icons/heart.svg
     * @default null
     */
    get icon() {
        return this._icon;
    }
    set icon(value) {
        const oldValue = this._icon;
        if (oldValue !== value) {
            this.deferIconReady();
            this._icon = value;
            requestAnimationFrame(() => this.updateRenderer());
            this.requestUpdate('icon', oldValue);
        }
    }
    /**
     * The icon template to render
     */
    get template() {
        return this._template;
    }
    set template(value) {
        if (this._template !== value) {
            this._template = value;
            this.requestUpdate();
        }
        this.iconReady.resolve();
    }
    constructor() {
        super();
        this._icon = null;
        this._template = EmptyTemplate;
        this.iconReady = new Deferred();
        // `iconReady` resolves at this stage so that `updateComplete` would be resolvable
        // even in the case that `icon` attribute is missing.
        this.iconReady.resolve();
    }
    /**
     * Check if the icon map configuration has content
     * @returns icon map if exists
     */
    get iconMap() {
        return (this.icon && this.config?.icon.map[this.icon]) || null;
    }
    /**
     * Called after the component is first rendered
     * @param changedProperties Properties which have changed
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.setPrefix();
    }
    async getUpdateComplete() {
        const result = await super.getUpdateComplete();
        await this.iconReady.promise;
        return result;
    }
    /**
     * instantiate a new deferred promise for icon ready if it's not pending already
     * @returns {void}
     */
    deferIconReady() {
        if (this.iconReady.isPending()) {
            return;
        }
        this.iconReady = new Deferred();
    }
    /**
     * Check if the icon is valid to render
     * @returns false if icon value or icon map value is invalid
     */
    isIconValid() {
        if (!this._icon) {
            return false;
        }
        if (this.iconMap && !isBase64svg(this.iconMap) && !isUrl(this.iconMap)) {
            return false;
        }
        return true;
    }
    /**
     * Update the icon renderer
     * @returns {void}
     */
    updateRenderer() {
        if (!this.isIconValid()) {
            return this.clearIcon();
        }
        const iconProperty = this._icon;
        if (this.iconMap) {
            void this.loadAndRenderIcon(this.iconMap);
        }
        else if (isUrl(iconProperty) || IconLoader.isPrefixResolved) {
            void this.loadAndRenderIcon(iconProperty);
        }
        else {
            void this.loadAndRenderSpriteIcon(iconProperty);
        }
    }
    /**
     * Tries to load an icon from the url provided
     * and the renders this into the icon template.
     * @param src Source location of the svg icon.
     * @returns {void}
     */
    async loadAndRenderIcon(src) {
        const iconTemplateCacheItem = iconTemplateCache.get(src);
        if (!iconTemplateCacheItem) {
            iconTemplateCache.set(src, IconLoader.loadSVG(src).then((body) => svg `${unsafeSVG(body)}`));
            return this.loadAndRenderIcon(src); // Load again and await cache result
        }
        this.template = await iconTemplateCacheItem;
    }
    /**
     * Tries to load get an icon from the sprite url provided
     * and the renders this into the icon template.
     * @param iconName Name of the svg icon.
     * @returns {void}
     */
    async loadAndRenderSpriteIcon(iconName) {
        const iconTemplateCacheItem = iconTemplateCache.get(iconName);
        if (!iconTemplateCacheItem) {
            iconTemplateCache.set(iconName, SpriteLoader.loadSpriteSVG(iconName).then((body) => svg `${unsafeSVG(body)}`));
            return this.loadAndRenderIcon(iconName); // Load again and await cache result
        }
        this.template = await iconTemplateCacheItem;
    }
    /**
     * Get and cache CDN prefix
     * This is a private URL which is set in the theme
     * and should not be configured again via the variable.
     * @returns {void}
     */
    setPrefix() {
        // This prefix for individual icons allows supporting custom prefix of self-managed icons.
        if (IconLoader.isPrefixPending) {
            const CDNPrefix = this.getComputedVariable('--cdn-prefix');
            IconLoader.setCdnPrefix(CDNPrefix);
        }
        if (SpriteLoader.isPrefixPending) {
            const CDNSpritePrefix = this.getComputedVariable('--cdn-sprite-prefix');
            SpriteLoader.setCdnPrefix(CDNSpritePrefix);
        }
    }
    /**
     * Clears SVG body from the icon template
     * @returns {void}
     */
    clearIcon() {
        this.template = EmptyTemplate;
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return this.template;
    }
};
__decorate([
    consume({ context: efConfig, subscribe: true })
], Icon.prototype, "config", void 0);
__decorate([
    property({ type: String, reflect: true })
], Icon.prototype, "icon", null);
Icon = __decorate([
    customElement('ef-icon')
], Icon);
export { Icon };
