import { __decorate } from "tslib";
import { ControlElement, css, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { createRef, ref } from '@refinitiv-ui/core/directives/ref.js';
import { isElementOverflown } from '@refinitiv-ui/utils/element.js';
import '../checkbox/index.js';
import '../icon/index.js';
import { registerOverflowTooltip } from '../tooltip/index.js';
import { VERSION } from '../version.js';
const isAllWhitespaceTextNode = (node) => node.nodeType === document.TEXT_NODE && !node.textContent?.trim();
/**
 * Used as a basic building block to compose complex custom elements,
 * additionally it can be used by applications
 * to create simple menus or navigation panels.
 *
 * @attr {string} value - The content of this attribute represents the value of the item.
 * @prop {string} [value=""] - The content of this attribute represents the value of the item.
 *
 * @attr {boolean} disabled - Set disabled state.
 * @prop {boolean} [disabled=false] - Set disabled state.
 *
 * @slot left - Used to render the content on the left of the label.
 * @slot right - Used to render the content on the right of the label.
 */
let Item = class Item extends ControlElement {
    constructor() {
        super(...arguments);
        /**
         * The text for the label indicating the meaning of the item.
         * By having both `label` and content, content always takes priority
         */
        this.label = null;
        /**
         * If defined value can be `text`, `header` or `divider`
         * @type {ItemType | null}
         */
        this.type = null;
        /**
         * Set the icon name from the ef-icon list
         */
        this.icon = null;
        /**
         * Indicates that the item is selected
         */
        this.selected = false;
        /**
         * Is the item part of a multiple selection
         */
        this.multiple = false;
        /**
         * Highlight the item
         */
        this.highlighted = false;
        /**
         * The`subLabel` property represents the text beneath the label.
         * By having both `subLabel` and content, content always takes priority
         */
        this.subLabel = null;
        /**
         * Specifies which element an item is bound to
         */
        this.for = null;
        /**
         * Reference to the label element
         */
        this.labelRef = createRef();
        /**
         * Reference to the subLabel element
         */
        this.subLabelRef = createRef();
        /**
         * Reference to the slot element
         */
        this.slotRef = createRef();
        /**
         * True, if there is no slotted content
         */
        this.isSlotEmpty = true;
        /**
         * Checks slotted children nodes and updates component to refresh label and sub-label templates.
         * @param event slotchange
         * @returns {void}
         */
        this.checkSlotChildren = (event) => {
            const slot = event.target;
            this.isSlotEmpty = !slot.assignedNodes().filter((node) => !this.isIgnorable(node)).length;
            this.requestUpdate();
        };
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * `CSSResultGroup` that will be used to style the host,
     * slotted children and the internal template of the element.
     * @returns CSS template
     */
    static get styles() {
        return css `
      :host {
        display: flex;
        align-items: center;
      }
      [part='checkbox'] {
        pointer-events: none;
      }
      [part='left'],
      [part='right'] {
        display: flex;
        align-items: center;
      }
      [part='center'] {
        flex: 1;
      }
      :host([type='divider']) > * {
        display: none;
      }
    `;
    }
    /**
     * @param node that should be checked
     * @returns whether node can be ignored.
     */
    isIgnorable(node) {
        return node.nodeType === document.COMMENT_NODE || isAllWhitespaceTextNode(node);
    }
    /**
     * Handles aria-selected or aria-checked when toggle between single and multiple selection mode
     * @returns {void}
     **/
    multipleChanged() {
        this.removeAttribute(this.multiple ? 'aria-selected' : 'aria-checked');
        this.selectedChanged();
    }
    /**
     * Handles aria when selected state changes
     * @returns {void}
     */
    selectedChanged() {
        this.setAttribute(this.multiple ? 'aria-checked' : 'aria-selected', String(this.selected));
    }
    /**
     * Control State behaviour will update tabindex based on the property
     * @returns {void}
     */
    typeChanged() {
        const noInteraction = this.type === 'header' || this.type === 'divider' || this.disabled;
        if (noInteraction) {
            this.disableFocus();
        }
        else if (!this.disabled) {
            this.enableFocus();
        }
    }
    /**
     * Called after the component is first rendered
     * @param changedProperties Properties which have changed
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        registerOverflowTooltip(this, () => this.getItemContent(), () => this.isItemOverflown());
    }
    /**
     * Invoked before update() to compute values needed during the update.
     * @param changedProperties changed properties
     * @returns {void}
     */
    willUpdate(changedProperties) {
        if (changedProperties.has('type')) {
            this.typeChanged();
        }
        if (changedProperties.has('multiple')) {
            this.multipleChanged();
        }
        else if (changedProperties.has('selected')) {
            this.selectedChanged();
        }
    }
    /**
     * Get Item content
     * @returns return item content from slot or label and sub-label
     */
    getItemContent() {
        if (this.isSlotEmpty) {
            let text = '';
            if (this.label) {
                text += this.label;
            }
            if (this.subLabel) {
                text += text ? ` (${this.subLabel})` : this.subLabel;
            }
            return text;
        }
        else {
            return this.slotContent;
        }
    }
    /**
     * Get element overflown
     * @param element Target element
     * @returns return true if element is overflown.
     */
    isItemElementOverflown(element) {
        return element ? isElementOverflown(element) : false;
    }
    /**
     * Get item overflown
     * @returns return true if an item is overflown.
     */
    isItemOverflown() {
        return (this.isItemElementOverflown(this.labelRef.value) || this.isItemElementOverflown(this.subLabelRef.value));
    }
    /**
     * Get icon template if icon attribute is defined
     */
    get iconTemplate() {
        return this.icon !== null && this.icon !== undefined
            ? html `<ef-icon part="icon" .icon="${this.icon}"></ef-icon>`
            : undefined;
    }
    /**
     * Get subLabel template if it is defined and no slot content present
     */
    get subLabelTemplate() {
        return html `<div part="sub-label" ${ref(this.subLabelRef)}>${this.subLabel}</div>`;
    }
    /**
     * Get label template if it is defined and no slot content present
     */
    get labelTemplate() {
        return html `${this.label}`;
    }
    /**
     * Get slot content
     */
    get slotContent() {
        const nodes = this.slotRef.value?.assignedNodes() || [];
        return nodes
            .map((node) => node.textContent)
            .join(' ')
            .trim();
    }
    /**
     * Get template for `for` attribute.
     * This is usually used with menus when an item needs to reference an element
     */
    get forTemplate() {
        return this.for ? html `<ef-icon icon="right"></ef-icon>` : undefined;
    }
    /**
     * Get template for `multiple` attribute.
     * This is usually used with lists, when an item can be part of a multiple selection
     */
    get multipleTemplate() {
        const multiple = this.multiple && (!this.type || this.type === 'text');
        return multiple
            ? html `<ef-checkbox part="checkbox" .checked="${this.selected}" tabindex="-1"></ef-checkbox>`
            : undefined;
    }
    /**
     * Return true if the item can be highlighted. True if not disabled and type is Text
     * @prop {boolean} highlightable
     * @default true
     * @returns whether element is highlightable
     */
    get highlightable() {
        return !this.disabled && this.type !== 'header' && this.type !== 'divider';
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @returns Render template
     */
    render() {
        return html `
      <div part="left">
        ${this.iconTemplate} ${this.multipleTemplate}
        <slot name="left"></slot>
      </div>
      <div part="center" ${ref(this.labelRef)}>
        ${this.label && this.isSlotEmpty ? this.labelTemplate : undefined}
        <slot ${ref(this.slotRef)} @slotchange="${this.checkSlotChildren}"></slot>
        ${this.subLabel && this.isSlotEmpty ? this.subLabelTemplate : undefined}
      </div>
      <div part="right">
        <slot name="right"></slot>
        ${this.forTemplate}
      </div>
    `;
    }
};
__decorate([
    property({ type: String })
], Item.prototype, "label", void 0);
__decorate([
    property({ type: String, reflect: true })
], Item.prototype, "type", void 0);
__decorate([
    property({ type: String, reflect: true })
], Item.prototype, "icon", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Item.prototype, "selected", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Item.prototype, "multiple", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Item.prototype, "highlighted", void 0);
__decorate([
    property({ type: String, reflect: true, attribute: 'sub-label' })
], Item.prototype, "subLabel", void 0);
__decorate([
    property({ type: String, reflect: true })
], Item.prototype, "for", void 0);
Item = __decorate([
    customElement('ef-item')
], Item);
export { Item };
