import { __decorate } from "tslib";
import { BasicElement, css, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { styleMap } from '@refinitiv-ui/core/directives/style-map.js';
import { addTooltipCondition, removeTooltipCondition } from '../tooltip/index.js';
import { VERSION } from '../version.js';
/**
 * Configuration object
 * for mutations observers
 */
const observerOptions = {
    subtree: true,
    childList: true,
    characterData: true
};
/**
 * Reusable SPACE
 */
const _ = ' ';
/**
 * Displays a text with alternative truncation
 */
let Label = class Label extends BasicElement {
    constructor() {
        super(...arguments);
        /**
         * Limit the number of lines before truncating
         */
        this.lineClamp = 0;
        /**
         * Set error state
         */
        this.error = false;
        /**
         * Set warning state
         */
        this.warning = false;
        /**
         * Trimmed chunks of textual content
         */
        this.chunks = [];
        /**
         * Mutation Observer used to detect changes in the Light DOM
         */
        this.mutationObserver = new MutationObserver(() => this.recalculate(true));
        /**
         * Render used to display the tooltip
         * @returns Tooltip text
         */
        this.tooltipRenderer = () => this.text;
        /**
         * Condition used to display the tooltip
         * @param target Tooltip target
         * @returns Whether the tooltip should be shown or not.
         */
        this.tooltipCondition = (target) => this.shouldShowTooltip(target);
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @returns CSS template
     */
    static get styles() {
        return css `
      :host {
        display: inline-flex;
        max-width: 100%;
      }
      .split {
        height: 1.2em;
        line-height: 1.2em;
        margin-top: -0.1em;
        flex: 0 1 auto;
        min-width: 0;
        overflow: hidden;
        white-space: nowrap;
      }
      .left {
        word-break: break-all;
        white-space: normal;
      }
      .center {
        flex: 0 100 auto;
      }
      .right {
        direction: rtl;
        text-overflow: ellipsis;
      }
      .clamp {
        overflow: hidden;
        -webkit-box-orient: vertical;
        display: -webkit-inline-box;
        text-overflow: ellipsis;
        position: relative;
        overflow-wrap: break-word;
      }
    `;
    }
    /**
     * @override
     */
    connectedCallback() {
        super.connectedCallback();
        addTooltipCondition(this.tooltipCondition, this.tooltipRenderer);
        this.mutationObserver.observe(this, observerOptions);
        this.recalculate();
    }
    /**
     * @override
     */
    disconnectedCallback() {
        super.disconnectedCallback();
        removeTooltipCondition(this.tooltipCondition);
        this.mutationObserver.disconnect();
    }
    /**
     * Decides whether the tooltip should b shown
     * @param tooltipTarget Target element passed by the tooltip condition
     * @returns True if the tooltip should be shown
     */
    shouldShowTooltip(tooltipTarget) {
        const targetMatches = tooltipTarget === this;
        const part = this.renderRoot.firstElementChild;
        if (targetMatches && part) {
            return part.clientHeight !== part.scrollHeight || part.clientWidth !== part.scrollWidth;
        }
        return false;
    }
    /**
     * Handles any modifications to the internal HTML
     * @param [mutation=false] is the request from a mutation event ? ( reserved for future used )
     * @returns {void}
     */
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    recalculate(mutation = false) {
        const oldValue = this.text;
        const raw = this.textContent || '';
        this.chunks = raw
            .split(_)
            .map((chunk) => chunk.trim())
            .filter((chunk) => chunk);
        const newValue = this.text;
        if (oldValue !== newValue) {
            this.requestUpdate('text', oldValue);
        }
    }
    /**
     * Returns cleaned version of `this.textContent`.
     */
    get text() {
        return this.chunks.join(_);
    }
    /**
     * Default template
     */
    get truncateTemplate() {
        const words = this.chunks;
        const left = [];
        const right = [];
        const isSingleWord = words.length === 1;
        if (isSingleWord) {
            const word = words[0];
            const split = Math.round(word.length / 2);
            left.push(word.substring(0, split));
            right.push(word.substring(split));
        }
        else {
            const split = Math.round(words.length / 2);
            for (let i = 0; i < words.length; i += 1) {
                (i < split ? left : right).push(words[i]);
            }
        }
        const leftPart = html `<div class="split left">${left.join(_)}</div>`;
        const centerPart = isSingleWord ? undefined : html `<div class="split center">&nbsp;</div>`;
        const rightPart = right.length
            ? html `<div class="split right"><span dir="ltr">${right.join(_)}</span></div>`
            : undefined;
        return html `${leftPart}${centerPart}${rightPart}`;
    }
    /**
     * Template for when line clamp is set
     */
    get clampTemplate() {
        const styles = {
            lineClamp: `${this.lineClamp}`,
            '-webkit-line-clamp': `${this.lineClamp}`,
            wordBreak: this.lineClamp === 1 ? 'break-all' : ''
        };
        return html ` <span class="clamp" style="${styleMap(styles)}">${this.text}</span> `;
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return this.lineClamp ? this.clampTemplate : this.truncateTemplate;
    }
};
__decorate([
    property({ type: Number, attribute: 'line-clamp' })
], Label.prototype, "lineClamp", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Label.prototype, "error", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Label.prototype, "warning", void 0);
Label = __decorate([
    customElement('ef-label')
], Label);
export { Label };
