import { __decorate } from "tslib";
import { BasicElement, css, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import '@refinitiv-ui/phrasebook/locale/en/notification.js';
import { translate } from '@refinitiv-ui/translate';
import '../../icon/index.js';
import { VERSION } from '../../version.js';
/**
 * Used to show informative content when something happens in the application
 *
 * @fires collapsed - Fired when notification is collapsed.
 * @fires dismiss - Fired when the user taps close button to dismiss notification. The event is not triggered if dismiss is done programmatically.
 *
 */
let Notification = class Notification extends BasicElement {
    constructor() {
        super(...arguments);
        /**
         * Default role of the element
         */
        this.defaultRole = 'alert';
        /**
         * The message to show in the notification.
         */
        this.message = '';
        /**
         * Notification style: Confirm
         */
        this.confirm = false;
        /**
         * Notification style: Warning
         */
        this.warning = false;
        /**
         * Notification style: Error
         */
        this.error = false;
        /**
         * Toggles the collapsed state.
         */
        this.collapsed = false;
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * On first updated lifecycle
     * @param changedProperties changed property
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.addEventListener('animationend', this.onAnimationEnd);
    }
    /**
     * Dismisses the notification, firing a `dismiss` event and collapsing the notification.
     * @returns {void}
     */
    dismiss() {
        const event = new CustomEvent('dismiss', {
            bubbles: false,
            cancelable: true
        });
        // do action only if it was not prevented by a handler
        if (this.dispatchEvent(event)) {
            this.collapsed = true;
        }
    }
    /**
     * Event handler for the clear button.
     * @param event event object
     * @returns {void}
     * @private
     */
    onClearClick(event) {
        event.stopPropagation();
        this.dismiss();
    }
    /**
     * Event handler for when animation end.
     * @returns {void}
     */
    onAnimationEnd() {
        if (this.collapsed) {
            this.dispatchEvent(new CustomEvent('collapsed', {
                bubbles: false,
                cancelable: false
            }));
        }
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     *
     * @returns CSS template
     */
    static get styles() {
        return css `
      :host {
        display: block;
      }
      [part='label'] {
        color: red;
      }
    `;
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     *
     * @returns {TemplateResult} Render template
     */
    render() {
        return html `
      <div part="container">
        <div part="content"><slot>${this.message}</slot></div>
        <ef-icon
          part="clear"
          icon="cross"
          role="button"
          aria-description="${this.t('CLOSE')}"
          @click="${this.onClearClick.bind(this)}"
        ></ef-icon>
      </div>
    `;
    }
};
__decorate([
    property({ type: String })
], Notification.prototype, "message", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Notification.prototype, "confirm", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Notification.prototype, "warning", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Notification.prototype, "error", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Notification.prototype, "collapsed", void 0);
__decorate([
    translate()
], Notification.prototype, "t", void 0);
Notification = __decorate([
    customElement('ef-notification')
], Notification);
export { Notification };
