import { NotificationTray } from '../elements/notification-tray.js';
const NotificationType = {
    INFO: 'INFO',
    CONFIRM: 'CONFIRM',
    WARN: 'WARN',
    ERROR: 'ERROR'
};
// Default notification tray to use
let tray = null;
/**
 * Tries to reuse a tray, if one is already available.
 * Otherwise, it will create a new tray to use.
 *
 * @param name tag name of the tray
 * @param forName notification tag name
 * @returns {void} notification tray
 */
const getTray = (name, forName) => {
    const existTray = document.querySelector(name + '[for="' + forName + '"]');
    if (existTray) {
        tray = existTray;
    }
    else {
        tray = new NotificationTray();
        tray.setAttribute('for', forName);
        tray.attach = 'top'; // attach to top
    }
};
/**
 * Connects the default tray to the document.
 * @returns {void}
 */
const connect = () => {
    if (document.body && tray) {
        document.body.appendChild(tray);
    }
    else {
        document.addEventListener('DOMContentLoaded', function () {
            connect();
        });
    }
};
/**
 * Prepares the default tray making sure it is available in the DOM.
 * @returns {void}
 */
const prepareTray = () => {
    getTray('ef-notification-tray', 'ef-notification');
    if (document.body && tray && tray.parentElement !== document.body) {
        connect();
    }
};
/**
 * Creates a notification and pushes it into the default tray.
 *
 * @param options notification task options
 * @returns instance of the `Notification`.
 */
const notify = (options) => {
    prepareTray();
    const el = document.createElement('ef-notification');
    el.message = options.message;
    el.confirm = options.type === NotificationType.CONFIRM;
    el.warning = options.type === NotificationType.WARN;
    el.error = options.type === NotificationType.ERROR;
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    tray.push(el, options);
    return el;
};
/**
 * Show an info notification (default)
 * @param message Message to show in the notification.
 * @param duration Duration the notification should be displayed for.
 * @returns instance of the `Notification`.
 */
const info = (message, duration) => {
    return notify({
        message: message,
        duration: duration,
        type: NotificationType.INFO
    });
};
/**
 * Show a confirmation notification
 * @param message Message to show in the notification.
 * @param duration Duration the notification should be displayed for.
 * @returns instance of the `Notification`.
 */
const confirm = (message, duration) => {
    return notify({
        message: message,
        duration: duration,
        type: NotificationType.CONFIRM
    });
};
/**
 * Show a warning notification
 * @param message Message to show in the notification.
 * @param duration Duration the notification should be displayed for.
 * @returns instance of the `Notification`.
 */
const warn = (message, duration) => {
    return notify({
        message: message,
        duration: duration,
        type: NotificationType.WARN
    });
};
/**
 * Show an error notification
 * @param message Message to show in the notification.
 * @param duration Duration the notification should be displayed for.
 * @returns instance of the `Notification`.
 */
const error = (message, duration) => {
    return notify({
        message: message,
        duration: duration,
        type: NotificationType.ERROR
    });
};
export { info, confirm, warn, error };
