import { __decorate } from "tslib";
import { html, unsafeCSS } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { state } from '@refinitiv-ui/core/decorators/state.js';
import '@refinitiv-ui/phrasebook/locale/en/password-field.js';
import { translate } from '@refinitiv-ui/translate';
import { VISUALLY_HIDDEN_STYLE } from '@refinitiv-ui/utils/accessibility.js';
import '../icon/index.js';
import { TextField } from '../text-field/index.js';
import { deregisterOverflowTooltip } from '../tooltip/index.js';
/**
 * A form control element for password.
 *
 * @fires value-changed - Fired when user commits a value change. The event is not triggered if `value` property is changed programmatically.
 * @fires error-changed - Fired when user inputs an invalid value. The event is not triggered if `error` property is changed programmatically.
 *
 * @attr {boolean} disabled - Set disabled state
 * @prop {boolean} [disabled=false] - Set disabled state
 *
 * @attr {boolean} error - Set error state
 * @prop {boolean} [error=false] - Set error state
 *
 * @attr {string} pattern - Set regular expression for input validation
 * @prop {string} [pattern=""] - Set regular expression for input validation
 *
 * @attr {string} placeholder - Set placeholder text
 * @prop {string} [placeholder=""] - Set placeholder text
 *
 * @attr {boolean} readonly - Set readonly state
 * @prop {boolean} [readonly=false] - Set readonly state
 *
 * @attr {boolean} transparent - Disables all other states and border/background styles.
 * @prop {boolean} [transparent=false] - Disables all other states and border/background styles.
 *
 * @attr {boolean} warning - Set warning state
 * @prop {boolean} [warning=false] - Set warning state
 *
 * @attr {string} value - Input's value
 * @prop {string} [value=""] - Input's value
 *
 * @attr {boolean} clears - Show clears button
 * @prop {boolean} [clears=false] - Show clears button
 */
let PasswordField = class PasswordField extends TextField {
    constructor() {
        super(...arguments);
        /**
         * Set character max limit
         */
        // override to merely fix missing attribute from component's doc
        this.maxLength = null;
        /**
         * Set character min limit
         */
        // override to merely fix missing attribute from component's doc
        this.minLength = null;
        /**
         * Defines whether password is visible or not
         */
        this.isPasswordVisible = false;
        /**
         * live region content presenting password field visibility state
         */
        this.liveRegionContent = '';
    }
    /**
     * A `CSSResultGroup` that will be used to style the host,
     * slotted children and the internal template of the element.
     * @returns CSS template
     */
    static get styles() {
        return [super.styles, unsafeCSS(VISUALLY_HIDDEN_STYLE)];
    }
    /**
     * Called when the element’s DOM has been updated and rendered for the first time
     * @param changedProperties Properties that has changed
     * @return shouldUpdate
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        // password shouldn't display value on tooltip when value is overflow
        deregisterOverflowTooltip(this);
    }
    /**
     * Returns `true` if the element input is valid; otherwise, returns `false`.
     * @returns element input validity
     */
    checkValidity() {
        return super.checkValidity();
    }
    /**
     * Validate the element input and mark it as error if its input is invalid.
     * @returns `true` if the element input is valid; otherwise, returns `false`.
     */
    reportValidity() {
        return super.reportValidity();
    }
    /**
     * Decorate `<input>` element with common properties extended from text-field:
     * type="text|password" - text if password is visible
     * @returns template map
     */
    get decorateInputMap() {
        return {
            ...super.decorateInputMap,
            type: this.isPasswordVisible ? 'text' : 'password'
        };
    }
    /**
     * Renders icon element
     * @returns {void}
     */
    renderIcon() {
        return html `
      <ef-icon
        part="icon"
        role="button"
        tabindex="0"
        aria-pressed="${this.isPasswordVisible}"
        aria-label="${this.t('SHOW_PASSWORD')}"
        icon=${this.isPasswordVisible ? 'eye-off' : 'eye'}
        ?readonly="${this.readonly}"
        ?disabled="${this.disabled}"
        @tap="${this.onTogglePasswordTap}"
        @focus="${this.updateLiveRegionContent}"
        @blur="${this.updateLiveRegionContent}"
      ></ef-icon>
      <div part="live-region" aria-live="polite" class="visually-hidden">${this.liveRegionContent}</div>
    `;
    }
    /**
     * update live region content describing password visibility state
     * @param event event trigging live region content update
     * @return void
     */
    updateLiveRegionContent(event) {
        this.liveRegionContent =
            event.type === 'blur'
                ? ''
                : this.isPasswordVisible
                    ? this.t('SHOW_PASSWORD_ON')
                    : this.t('SHOW_PASSWORD_OFF');
    }
    /**
     * Handle tap events of toggle password icon
     * @param event custom event
     * @returns {void}
     */
    onTogglePasswordTap(event) {
        this.togglePasswordVisibility();
        this.updateLiveRegionContent(event);
    }
    /**
     * Toggles password visibility state
     * @return void
     */
    togglePasswordVisibility() {
        this.isPasswordVisible = !this.isPasswordVisible;
    }
};
__decorate([
    property({ type: Number, attribute: 'maxlength', reflect: true })
], PasswordField.prototype, "maxLength", void 0);
__decorate([
    property({ type: Number, attribute: 'minlength', reflect: true })
], PasswordField.prototype, "minLength", void 0);
__decorate([
    translate({ scope: 'ef-password-field' })
], PasswordField.prototype, "t", void 0);
__decorate([
    state()
], PasswordField.prototype, "isPasswordVisible", void 0);
__decorate([
    state()
], PasswordField.prototype, "liveRegionContent", void 0);
PasswordField = __decorate([
    customElement('ef-password-field')
], PasswordField);
export { PasswordField };
