import { __decorate } from "tslib";
import { ControlElement, css, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { createRef, ref } from '@refinitiv-ui/core/directives/ref.js';
import { isElementOverflown } from '@refinitiv-ui/utils/element.js';
import '../icon/index.js';
import { registerOverflowTooltip } from '../tooltip/index.js';
import { VERSION } from '../version.js';
/**
 * A small button style component
 * which is used to show one or multiple selected item.
 * It is rarely used in the UI but inside other components to visualize multiple item selection item.
 * @attr {string} value - Value of pill
 * @prop {string} [value=""] - Value of pill
 *
 * @attr {boolean} readonly - Set readonly state
 * @prop {boolean} [readonly=false] - Set readonly state
 *
 * @attr {boolean} disabled - Set disabled state
 * @prop {boolean} [disabled=false] - Set disabled state
 *
 * @fires clear - Fired when the user taps on clear button.
 */
let Pill = class Pill extends ControlElement {
    constructor() {
        super(...arguments);
        /**
         * Element's role attribute for accessibility
         */
        this.defaultRole = 'button';
        /**
         * Set pill to clearable
         */
        this.clears = false;
        /**
         * Set pill to toggle mode
         */
        this.toggles = false;
        /**
         * Set active state
         */
        this.active = false;
        /**
         * Set property pressed true on tap start and false on tap end
         */
        this.pressed = false;
        /**
         * Reference to the close icon
         */
        this.closeIconRef = createRef();
        /**
         * Reference to the label element
         */
        this.labelRef = createRef();
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @return CSS template
     */
    static get styles() {
        return css `
      :host {
        display: inline-block;
        position: relative;
      }
      [part='content'] {
        text-overflow: ellipsis;
        white-space: nowrap;
        overflow: hidden;
      }
    `;
    }
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.addEventListener('tap', this.onTapHandler);
        this.addEventListener('tapstart', this.onStartPress);
        this.addEventListener('tapend', this.onEndPress);
        this.addEventListener('mouseleave', this.onEndPress);
        this.addEventListener('keydown', this.onKeyDown);
        registerOverflowTooltip(this, undefined, () => this.labelRef.value ? isElementOverflown(this.labelRef.value) : false);
    }
    /**
     * Compute property values that depend on other properties
     * and are used in the rest of the update process.
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    willUpdate(changedProperties) {
        super.willUpdate(changedProperties);
        if (changedProperties.has('toggles') || changedProperties.has('active')) {
            if (this.toggles) {
                this.setAttribute('aria-pressed', String(this.active));
            }
            else {
                this.removeAttribute('aria-pressed');
            }
        }
    }
    /**
     * Handles key down event
     * @param event Key down event object
     * @returns {void}
     */
    onKeyDown(event) {
        if (event.key === 'Delete' && this.clears && !this.readonly) {
            this.dispatchEvent(new CustomEvent('clear'));
        }
    }
    get closeTemplate() {
        return this.clears && !this.readonly
            ? html `<ef-icon
          ${ref(this.closeIconRef)}
          part="close"
          icon="cross"
          aria-hidden="true"
          @tap="${this.clear}"
        ></ef-icon>`
            : null;
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return html `
      <div ${ref(this.labelRef)} part="content" role="none">
        <slot>...</slot>
      </div>
      ${this.closeTemplate}
    `;
    }
    /**
     * change state of `pressed` property to be true if there is no close icon or pill is pressed directly
     * @param event tapstart event
     * @returns {void}
     */
    onStartPress(event) {
        if (this.couldBePressed(event)) {
            this.pressed = true;
        }
    }
    /**
     * @param event tapstart
     * @returns true if element property pressed could be set
     */
    couldBePressed(event) {
        const closeIconEl = this.closeIconRef.value;
        return !this.readonly && (!closeIconEl || !event.composedPath().includes(closeIconEl));
    }
    /**
     * change state of `pressed` property to be false if mouse leave pill or tap is end on pill
     * @returns {void}
     */
    onEndPress() {
        if (this.pressed) {
            this.pressed = false;
        }
    }
    /**
     * handle when `clears` icon is tapped
     * @returns {void}
     */
    onTapHandler() {
        if (this.toggles && !this.readonly) {
            this.active = !this.active;
        }
    }
    /**
     * @param event event from close button
     * @returns {void}
     */
    clear(event) {
        event.stopPropagation();
        /**
         * Fires when click on cross occurs.
         */
        this.dispatchEvent(new CustomEvent('clear'));
    }
};
__decorate([
    property({ type: Boolean, reflect: true })
], Pill.prototype, "clears", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Pill.prototype, "toggles", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Pill.prototype, "active", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Pill.prototype, "pressed", void 0);
Pill = __decorate([
    customElement('ef-pill')
], Pill);
export { Pill };
