import { __decorate } from "tslib";
import { BasicElement, css, html, nothing } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { query } from '@refinitiv-ui/core/decorators/query.js';
import '../layout/index.js';
import { VERSION } from '../version.js';
/**
 * Provides an app layout with sidebar.
 * There are 4 sections that can be slotted a component in.
 *
 * @slot sidebar-header - Sidebar header.
 * @slot sidebar-content - Content of sidebar.
 * @slot main-header - Main header.
 * @slot main-content - Content of main section.
 */
let SidebarLayout = class SidebarLayout extends BasicElement {
    constructor() {
        super(...arguments);
        /**
         * Set to hide sidebar
         */
        this.collapsed = false;
        /**
         * Set sidebar position to left or right
         */
        this.sidebarPosition = 'left';
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @return CSS template
     */
    static get styles() {
        return css `
      :host {
        display: block;
      }

      [part='container'] {
        height: 100%;
      }

      ef-layout ::slotted(ef-panel) {
        width: 100%;
        min-height: 100%;
      }

      .content {
        height: 100%;
      }

      [part='sidebar'] {
        width: var(--sidebar-width);
      }

      :host([collapsed]:not([sidebar-position])) [part='sidebar'],
      :host([collapsed][sidebar-position='left']) [part='sidebar'] {
        margin-left: calc(var(--sidebar-width) * -1);
      }

      :host([collapsed][sidebar-position='right']) [part='sidebar'] {
        margin-right: calc(var(--sidebar-width) * -1);
      }
    `;
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return html `
      <ef-layout flex nowrap part="container">
        <ef-layout flex container part="sidebar" size="${this.sidebarWidth || nothing}">
          <ef-layout size="auto">
            <slot name="sidebar-header"></slot>
          </ef-layout>
          <ef-layout class="content" scrollable basis="auto">
            <slot name="sidebar-content"></slot>
          </ef-layout>
        </ef-layout>

        <ef-layout flex container basis="100%" part="main">
          <ef-layout size="auto">
            <slot name="main-header"></slot>
          </ef-layout>
          <ef-layout class="content" scrollable basis="auto">
            <slot name="main-content"></slot>
          </ef-layout>
        </ef-layout>
      </ef-layout>
    `;
    }
    /**
     * @override
     * @returns {void}
     */
    updated(changedProperties) {
        super.updated(changedProperties);
        if (changedProperties.has('sidebarWidth')) {
            this.updateVariable('--sidebar-width', this.sidebarWidth);
        }
    }
};
__decorate([
    property({ type: String, attribute: 'sidebar-width' })
], SidebarLayout.prototype, "sidebarWidth", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], SidebarLayout.prototype, "collapsed", void 0);
__decorate([
    property({ type: String, reflect: true, attribute: 'sidebar-position' })
], SidebarLayout.prototype, "sidebarPosition", void 0);
__decorate([
    query('[part=sidebar]')
], SidebarLayout.prototype, "sidebar", void 0);
SidebarLayout = __decorate([
    customElement('ef-sidebar-layout')
], SidebarLayout);
export { SidebarLayout };
