import { __decorate } from "tslib";
import { BasicElement, css, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { query } from '@refinitiv-ui/core/decorators/query.js';
import '../button/index.js';
import '../layout/index.js';
import { Tab } from '../tab/index.js';
import { VERSION } from '../version.js';
import { tweenAnimate } from './helpers/animate.js';
const BAR_TRAVEL_DISTANCE = 150; // scroll distance
/**
 * Container for tabs
 *
 * @fires value-changed - Fired when the user changes an active tab. The event is not triggered if `value` property is changed programmatically.
 */
let TabBar = class TabBar extends BasicElement {
    constructor() {
        super(...arguments);
        this.defaultRole = 'tablist';
        /**
         * Specify tab's horizontal alignment
         */
        this.alignment = 'left';
        /**
         * Use level styling from theme
         */
        this.level = '1';
        /**
         * Use to switch from horizontal to vertical layout.
         */
        this.vertical = false;
        /**
         * Internal value of tab bar.
         * Controlled by public setter and getter
         */
        this._value = '';
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @returns CSS template
     */
    static get styles() {
        return css `
      :host {
        display: flex;
      }
      :host([alignment='center']) {
        justify-content: center;
      }
      :host([alignment='right']) {
        justify-content: flex-end;
      }
      :host [part='content'] {
        flex-grow: 0;
      }
    `;
    }
    /**
     * Value of tab-bar, derived from value of an active tab.
     * @param value Element value
     * @default -
     */
    set value(value) {
        value = typeof value === 'string' ? value : String(value);
        const oldValue = this._value;
        if (value !== oldValue && this.isValidValue(value)) {
            this._value = value;
            this.activateTab(value);
            this.requestUpdate('value', oldValue);
        }
    }
    get value() {
        return this._value;
    }
    /**
     * Called after the element’s DOM has been updated the first time.
     * register scroll event on content element to toggle scroll button
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.content.addEventListener('scroll', () => {
            // Clear our timeout throughout the scroll
            window.clearTimeout(this.isScrolling);
            // Set a timeout to run after scrolling ends
            this.isScrolling = window.setTimeout(() => {
                this.toggleScrollButton(this.content.clientWidth);
            }, 66); // equal 15 fps for compatibility
        });
        this.addEventListener('tap', this.onTap);
        this.addEventListener('keydown', this.onKeyDown);
    }
    /**
     * Called when the element’s DOM has been updated and rendered
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    updated(changedProperties) {
        if (changedProperties.has('level')) {
            this.setLevel();
        }
        super.updated(changedProperties);
    }
    /**
     * Handles content's container resize
     * @param event Resize event
     * @returns {void}
     */
    handleResize(event) {
        if (!this.vertical) {
            this.toggleScrollButton(event.detail.width);
        }
    }
    /**
     * Return true if incoming value matches one of the existing tabs
     * @param value Value to check
     * @returns true if incoming value matches one of the existing tabs
     */
    isValidValue(value) {
        const tabList = this.getFocusableTabs();
        return tabList.some((tab) => this.getTabValue(tab) === value);
    }
    /**
     * On *user-interaction* set the value and notify.
     * @param value New value
     * @returns {void}
     */
    setValueAndNotify(value) {
        if (this.value !== value) {
            this.value = value;
            this.notifyPropertyChange('value', value);
        }
    }
    /**
     * When the slot changes, set the level, toggle the scroll button, and set the value
     * @returns {void}
     */
    onSlotChange() {
        const tabList = this.getFocusableTabs();
        if (tabList.length < 1) {
            return;
        }
        this.setLevel();
        // get tab value from active tab
        const activeTab = tabList.find((tab) => tab.active) || tabList[0];
        if (activeTab) {
            this.value = this.getTabValue(activeTab);
        }
        this.manageTabIndex();
    }
    /**
     * Mark tab as active
     * @param value value of tab to select
     * @returns {void}
     */
    activateTab(value) {
        if (!value) {
            return;
        }
        let hasActiveTab = false;
        const tabList = this.getTabElements(); // get all tab elements include disabled tab
        tabList.forEach((tab) => {
            const tabValue = this.getTabValue(tab);
            // only mark tab as active once
            if (tabValue === value && !hasActiveTab && !tab.disabled) {
                tab.active = true;
                hasActiveTab = true;
            }
            else {
                tab.active = false;
            }
        });
    }
    /**
     * Set tab value and fires `tab-changed` event
     * @param event - Event
     * @returns {void}
     */
    onTap(event) {
        if (event.defaultPrevented) {
            return;
        }
        const element = event.target;
        if (element instanceof Tab) {
            const tabValue = this.getTabValue(element);
            this.setValueAndNotify(tabValue);
        }
    }
    /**
     * Get the value of a tab
     * @param tab - The tab element.
     * @returns The value of the tab.
     */
    getTabValue(tab) {
        return tab.value || (tab.hasAttribute('value') ? '' : this.getTabLabel(tab));
    }
    /**
     * Return the tab's label, or its textContent, or an empty string
     * @param tab - The tab element.
     * @returns The tab label.
     */
    getTabLabel(tab) {
        return tab.label || tab.textContent || '';
    }
    /**
     * Get Tab elements from slot
     * @returns the array of Tab
     */
    getTabElements() {
        const tabs = [];
        for (const child of this.children) {
            if (child instanceof Tab) {
                tabs.push(child);
            }
        }
        return tabs;
    }
    /**
     * Get focusable tab elements
     * @returns the array of focusable tab
     */
    getFocusableTabs() {
        return this.getTabElements().filter((tab) => !tab.disabled);
    }
    /**
     * Set tab level attribute accordingly
     * @returns {void}
     */
    setLevel() {
        const tabList = this.getTabElements(); // get all tab elements include disabled tab
        tabList?.forEach((tab) => {
            tab.level = this.level;
        });
    }
    /**
     * Hide/Show scroll button when element is overflow.
     * @param elementWidth width of element
     * @returns {void}
     */
    toggleScrollButton(elementWidth) {
        if (this.vertical) {
            return;
        }
        const { scrollLeft, scrollWidth } = this.content;
        const leftBtnStyle = scrollLeft > 0 ? 'flex' : 'none';
        const rightBtnStyle = scrollWidth - scrollLeft - elementWidth > 1 ? 'flex' : 'none';
        this.leftBtn.style.setProperty('display', leftBtnStyle);
        this.rightBtn.style.setProperty('display', rightBtnStyle);
    }
    /**
     * Update scroll position when clicked on left button
     * @returns {void}
     */
    handleScrollLeft() {
        const { scrollLeft } = this.content;
        const availableScrollLeft = scrollLeft;
        let endPosition = scrollLeft - BAR_TRAVEL_DISTANCE;
        // If the space available is less than one half lots of our desired distance, just move to the leftest
        if (availableScrollLeft < BAR_TRAVEL_DISTANCE * 1.5) {
            endPosition = 0;
        }
        tweenAnimate({ target: this.content, startPosition: scrollLeft, endPosition });
    }
    /**
     * Update scroll position when clicked on right button
     * @returns {void}
     */
    handleScrollRight() {
        const { scrollLeft, scrollWidth, clientWidth } = this.content;
        const availableScrollRight = scrollWidth - (scrollLeft + clientWidth);
        let endPosition = scrollLeft + BAR_TRAVEL_DISTANCE;
        // If the space available is less than one half lots of our desired distance, just move the whole amount
        if (availableScrollRight < BAR_TRAVEL_DISTANCE * 1.5) {
            endPosition = scrollLeft + availableScrollRight;
        }
        tweenAnimate({ target: this.content, startPosition: scrollLeft, endPosition });
    }
    /**
     * Focus and set active to tab
     * @param tab - The element that was clicked.
     * @return {void}
     */
    focusAndSetActiveTab(tab) {
        tab.focus();
        tab.scrollIntoView({ block: 'nearest' });
        const tabValue = this.getTabValue(tab);
        this.setValueAndNotify(tabValue);
    }
    /**
     * Navigate to first focusable tab of the tab bar
     * @returns {void}
     */
    first() {
        const tabList = this.getFocusableTabs();
        if (tabList.length <= 0) {
            return;
        }
        this.focusAndSetActiveTab(tabList[0]);
        this.rovingTabIndex(tabList[0], tabList);
    }
    /**
     * Navigate to last focusable tab of the tab bar
     * @returns {void}
     */
    last() {
        const tabList = this.getFocusableTabs();
        if (tabList.length <= 0) {
            return;
        }
        const lastTab = tabList[tabList.length - 1];
        this.focusAndSetActiveTab(lastTab);
        this.rovingTabIndex(lastTab, tabList);
    }
    /**
     * Navigate to next or previous focusable tab
     * @param direction up/next; down/previous
     * @returns {void}
     */
    navigateToSibling(direction) {
        const tabList = this.getFocusableTabs();
        if (tabList.length <= 0) {
            return;
        }
        const focusedTabIndex = tabList.findIndex((tab) => tab === document.activeElement);
        const nextTab = direction === 'next'
            ? tabList[focusedTabIndex + 1] || tabList[0]
            : tabList[focusedTabIndex - 1] || tabList[tabList.length - 1];
        this.focusAndSetActiveTab(nextTab);
        this.rovingTabIndex(nextTab, tabList);
    }
    /**
     * Handles key down event
     * @param event Key down event object
     * @returns {void}
     */
    onKeyDown(event) {
        if (event.defaultPrevented) {
            return;
        }
        switch (event.key) {
            case 'ArrowRight':
            case 'ArrowDown':
                this.navigateToSibling('next');
                break;
            case 'ArrowLeft':
            case 'ArrowUp':
                this.navigateToSibling('previous');
                break;
            case 'Home':
                this.first();
                break;
            case 'End':
                this.last();
                break;
            default:
                return;
        }
        event.preventDefault();
    }
    /**
     * Sets the tabindex to -1 for all tabs except the active tab.
     * @param target the tab to be focused
     * @param tabList Array of tabs that contains target
     * @returns {void}
     */
    rovingTabIndex(target, tabList) {
        tabList.forEach((tab) => {
            tab.tabIndex = -1;
        });
        target.tabIndex = 0;
    }
    /**
     * Set tabIndex to all tabs
     * @returns {void}
     */
    manageTabIndex() {
        const tabList = this.getFocusableTabs();
        if (tabList && tabList.length > 0) {
            const focusedTabIndex = tabList.findIndex((tab) => tab.active);
            this.rovingTabIndex(tabList[focusedTabIndex], tabList);
        }
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return html `
      ${!this.vertical
            ? html `<ef-button
            tabIndex="-1"
            aria-hidden="true"
            icon="left"
            part="left-btn"
            @tap=${this.handleScrollLeft}
          ></ef-button>`
            : null}
      <ef-layout part="content" @resize=${this.handleResize}>
        <slot @slotchange=${this.onSlotChange}></slot>
      </ef-layout>
      ${!this.vertical
            ? html `<ef-button
            tabIndex="-1"
            aria-hidden="true"
            icon="right"
            part="right-btn"
            @tap=${this.handleScrollRight}
          ></ef-button>`
            : null}
    `;
    }
};
__decorate([
    property({ type: String, reflect: true })
], TabBar.prototype, "alignment", void 0);
__decorate([
    property({ type: String, reflect: true })
], TabBar.prototype, "level", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], TabBar.prototype, "vertical", void 0);
__decorate([
    property({ type: String, attribute: false })
], TabBar.prototype, "value", null);
__decorate([
    query('[part="content"')
], TabBar.prototype, "content", void 0);
__decorate([
    query('[part="left-btn"]')
], TabBar.prototype, "leftBtn", void 0);
__decorate([
    query('[part="right-btn"]')
], TabBar.prototype, "rightBtn", void 0);
TabBar = __decorate([
    customElement('ef-tab-bar')
], TabBar);
export { TabBar };
