import { __decorate } from "tslib";
import { ControlElement, css, html } from '@refinitiv-ui/core';
import { triggerResize } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { state } from '@refinitiv-ui/core/decorators/state.js';
import { isSlotEmpty } from '@refinitiv-ui/utils/is-slot-empty.js';
import '../icon/index.js';
import '../label/index.js';
import { VERSION } from '../version.js';
/**
 * A building block for individual tab
 * @attr {boolean} disabled - Set disabled state
 * @prop {boolean} [disabled=false] - Set disabled state
 *
 * @attr {string} value - Tab's value
 * @prop {string} [value=""] - Tab's value
 *
 * @fires clear - Fired when the user clicks on clear button
 */
let Tab = class Tab extends ControlElement {
    constructor() {
        super(...arguments);
        this.defaultRole = 'tab';
        /**
         * Specify icon name to display in tab
         */
        this.icon = '';
        /**
         * Specify tab's label text
         */
        this.label = '';
        /**
         * Specify tab's sub-label text
         */
        this.subLabel = '';
        /**
         * Specify tab's active status
         */
        this.active = false;
        /**
         * Set tab to clearable
         */
        this.clears = false;
        /**
         * Limit the number of lines before truncating
         */
        this.lineClamp = 1;
        /**
         * Set tab to clearable on hover
         */
        this.clearsOnHover = false;
        /**
         * Use level styling from theme
         * @ignore
         */
        this.level = '1';
        /**
         * True, if there is slotted content
         */
        this.isSlotHasContent = false;
        /**
         * Run on default slot slotchange
         * @param event slotchange
         * @returns {void}
         */
        this.onSlotChange = (event) => {
            const slot = event.target;
            this.isSlotHasContent = !!slot.assignedNodes().length && isSlotEmpty(slot);
            // ensures that tab-bar fire resizeCallback when slot changes
            triggerResize();
        };
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @returns CSS template
     */
    static get styles() {
        return css `
      :host {
        display: inline-flex;
        flex-shrink: 0;
      }
    `;
    }
    /**
     * Called after the element’s DOM has been updated the first time.
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.addEventListener('keydown', this.onKeyDown);
    }
    /**
     * Compute property values that depend on other properties
     * and are used in the rest of the update process.
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    willUpdate(changedProperties) {
        super.willUpdate(changedProperties);
        if (changedProperties.has('active')) {
            this.setAttribute('aria-selected', this.active ? 'true' : 'false');
        }
        if (changedProperties.has('label')) {
            // ensures that tab-bar fire resizeCallback when label changes
            triggerResize();
        }
    }
    /**
     * Omitted lineClamp if subLabel is provided
     * @returns line Clamp value
     */
    getLineClamp() {
        return !this.lineClamp ? 0 : this.subLabel ? 1 : this.lineClamp;
    }
    /**
     * Handles key down event
     * @param event Key down event object
     * @returns {void}
     */
    onKeyDown(event) {
        if (event.defaultPrevented) {
            return;
        }
        if (event.key === 'Delete' && (this.clears || this.clearsOnHover)) {
            this.dispatchEvent(new CustomEvent('clear'));
        }
    }
    /**
     * @param event event from close button
     * @returns {void}
     */
    handleClickClear(event) {
        event.stopPropagation();
        /**
         * Fires when click on cross occurs
         */
        this.dispatchEvent(new CustomEvent('clear'));
    }
    /**
     * Show Close Button if allow clears
     * @returns close button template
     */
    get CloseTemplate() {
        return this.clears || this.clearsOnHover
            ? html `
          <div part="close-container">
            <ef-icon part="close" icon="cross" @tap="${this.handleClickClear}"></ef-icon>
          </div>
        `
            : null;
    }
    /**
     * Create ef-label template when label is true
     * @returns Label template
     */
    get LabelTemplate() {
        if (!this.label || this.isSlotHasContent) {
            return null;
        }
        return html ` <ef-label part="label" .lineClamp=${this.getLineClamp()}> ${this.label} </ef-label> `;
    }
    /**
     * Create ef-label template when subLabel is true
     * @returns SubLabel template
     */
    get SubLabelTemplate() {
        if (!this.subLabel || this.isSlotHasContent) {
            return null;
        }
        return html ` <ef-label part="sub-label" .lineClamp=${this.getLineClamp()}> ${this.subLabel} </ef-label> `;
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return html `
      ${this.icon ? html `<ef-icon icon=${this.icon} part="icon"></ef-icon>` : null}
      <div part="label-container">
        ${this.LabelTemplate} ${this.SubLabelTemplate}
        <slot @slotchange="${this.onSlotChange}"></slot>
      </div>
      ${this.CloseTemplate}
    `;
    }
};
__decorate([
    property({ type: String })
], Tab.prototype, "icon", void 0);
__decorate([
    property({ type: String })
], Tab.prototype, "label", void 0);
__decorate([
    property({ type: String, attribute: 'sub-label' })
], Tab.prototype, "subLabel", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Tab.prototype, "active", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Tab.prototype, "clears", void 0);
__decorate([
    property({ type: Number, reflect: true, attribute: 'line-clamp' })
], Tab.prototype, "lineClamp", void 0);
__decorate([
    property({ type: Boolean, reflect: true, attribute: 'clears-on-hover' })
], Tab.prototype, "clearsOnHover", void 0);
__decorate([
    property({ type: String, reflect: true })
], Tab.prototype, "level", void 0);
__decorate([
    state()
], Tab.prototype, "isSlotHasContent", void 0);
Tab = __decorate([
    customElement('ef-tab')
], Tab);
export { Tab };
