import { __decorate } from "tslib";
import { ControlElement, css, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { VERSION } from '../version.js';
/**
 * Return the attribute that converted from the property
 * Prevent empty string that reflected to attribute
 * @private
 * @param value value from the property
 * @returns string converted to attribute
 */
const emptyStringToNull = function (value) {
    return value || null;
};
/**
 * Form control that can toggle between 2 states
 *
 * @attr {boolean} readonly - Set readonly state
 * @prop {boolean} [readonly=false] - Set readonly state
 *
 * @attr {boolean} disabled - Set disabled state
 * @prop {boolean} [disabled=false] - Set disabled state
 *
 * @fires checked-changed - Fired when user interacts with the control to check or uncheck. The event is not triggered if `checked` property is changed programmatically.
 */
let Toggle = class Toggle extends ControlElement {
    constructor() {
        super(...arguments);
        this.defaultRole = 'switch';
        /**
         * Label of the checked state
         */
        this.checkedLabel = '';
        /**
         * Label of toggle
         */
        this.label = '';
        /**
         * Value of toggle
         */
        this.checked = false;
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * A `CSSResultGroup` that will be used
     * to style the host, slotted children
     * and the internal template of the element.
     * @returns CSS template
     */
    static get styles() {
        return css `
      :host {
        display: inline-block;
      }
    `;
    }
    /**
     * Called before update() to compute values needed during the update.
     * @param changedProperties Properties that has changed
     * @returns {void}
     */
    willUpdate(changedProperties) {
        super.willUpdate(changedProperties);
        if (changedProperties.has('checked')) {
            this.setAttribute('aria-checked', String(this.checked));
        }
    }
    /**
     * Invoked when the element is first updated. Implement to perform one time
     * work on the element after update.
     * @param changedProperties Map of changed properties with old values
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.addEventListener('tap', this.handleCheckedChange);
        this.addEventListener('keydown', this.handleKeyDown);
    }
    /**
     * Called when checked value changes and dispatch the event
     * @returns {void}
     */
    handleCheckedChange() {
        if (this.disabled || this.readonly) {
            return;
        }
        this.checked = !this.checked;
        this.notifyPropertyChange('checked', this.checked);
    }
    /**
     * Handle keyboard event for toggle action
     * @param event Keyboard event
     * @returns {void}
     */
    handleKeyDown(event) {
        if (event.key === 'Enter' || event.key === ' ') {
            this.handleCheckedChange();
        }
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @return Render template
     */
    render() {
        return html ` <div part="toggle">
      ${this.checked && this.checkedLabel ? this.checkedLabel : this.label}
    </div>`;
    }
};
__decorate([
    property({
        type: String,
        attribute: 'checked-label',
        reflect: true,
        converter: { toAttribute: emptyStringToNull }
    })
], Toggle.prototype, "checkedLabel", void 0);
__decorate([
    property({
        type: String,
        reflect: true,
        converter: { toAttribute: emptyStringToNull }
    })
], Toggle.prototype, "label", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], Toggle.prototype, "checked", void 0);
Toggle = __decorate([
    customElement('ef-toggle')
], Toggle);
export { Toggle };
