import { __decorate } from "tslib";
import { ControlElement, html } from '@refinitiv-ui/core';
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import '../../checkbox/index.js';
import '../../icon/index.js';
import { VERSION } from '../../version.js';
import { CheckedState } from '../managers/tree-manager.js';
const emptyTemplate = html ``;
/**
 * Displays a tree list item.
 * Groups display expansion arrows.
 */
let TreeItem = class TreeItem extends ControlElement {
    constructor() {
        super(...arguments);
        /**
         * Tree-item should not be focusable
         */
        this.defaultTabIndex = null;
        this.defaultRole = 'treeitem';
        /**
         * Checked state of the item
         */
        this.checkedState = CheckedState.UNCHECKED;
        /**
         * Is the item a parent and should it show an expansion toggle?
         */
        this.parent = false;
        /**
         * Display in multiple selection mode
         */
        this.multiple = false;
        /**
         * Expanded state of the item
         */
        this.expanded = false;
        /**
         * Depth of the item
         */
        this.depth = 0;
        /**
         * Label of the item
         */
        this.label = '';
        /**
         * Icon of the item
         */
        this.icon = '';
        /**
         * Highlighted state of the item.
         * This is for showing which item is currently being interacted with.
         */
        this.highlighted = false;
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * Template for rendering the indentation element
     */
    get indentTemplate() {
        return this.depth ? html `<div part="indent" style="width:${this.depth}em"></div>` : emptyTemplate;
    }
    /**
     * Template for rendering the toggle
     *
     * ! expand-toggle is required for automatically toggling expanded state
     */
    get toggleTemplate() {
        return html `
      <div
        expand-toggle
        part="toggle"
        style="pointer-events:all;visibility:${this.parent ? 'visible' : 'hidden'}"
      >
        <ef-icon part="toggle-icon${this.expanded ? ' toggle-icon-expanded' : ''}" icon="right"></ef-icon>
      </div>
    `;
    }
    /**
     * Template for rendering the checkbox
     */
    get checkboxTemplate() {
        if (!this.multiple) {
            return emptyTemplate;
        }
        return html `
      <ef-checkbox
        part="checkbox"
        tabindex="-1"
        .disabled="${this.disabled}"
        .readonly="${this.readonly}"
        .indeterminate="${this.indeterminate}"
        .checked="${this.checked}"
        style="pointer-events:none"
      ></ef-checkbox>
    `;
    }
    /**
     * Template for rendering the icon
     */
    get iconTemplate() {
        if (typeof this.icon === 'undefined') {
            return emptyTemplate;
        }
        return html `<ef-icon part="label-icon" icon="${this.icon}"></ef-icon>`;
    }
    /**
     * Is the item fully checked?
     */
    get checked() {
        return this.checkedState === CheckedState.CHECKED;
    }
    /**
     * Is the checked state indeterminate?
     */
    get indeterminate() {
        return this.checkedState === CheckedState.INDETERMINATE;
    }
    /**
     * Handles aria-checked and aria-selected when mode changes
     * aria-checked is used for multiple mode due to tri-state support
     * @returns {void}
     **/
    multipleChanged() {
        this.removeAttribute(this.multiple ? 'aria-selected' : 'aria-checked');
        this.checkedChanged();
    }
    /**
     * Handles selected and aria attribute changes
     * @returns {void}
     */
    checkedChanged() {
        switch (this.checkedState) {
            case CheckedState.CHECKED:
                this.setAttribute('selected', '');
                this.setAttribute(this.multiple ? 'aria-checked' : 'aria-selected', 'true');
                break;
            case CheckedState.INDETERMINATE:
                this.setAttribute('selected', 'indeterminate');
                this.setAttribute('aria-checked', 'mixed');
                break;
            default:
                this.removeAttribute('selected');
                // In single mode, only children nodes are selectable
                if (this.parent && !this.multiple) {
                    this.removeAttribute('aria-selected');
                }
                else {
                    this.setAttribute(this.multiple ? 'aria-checked' : 'aria-selected', 'false');
                }
                break;
        }
    }
    /**
     * Handles aria-expanded when expanded state changes
     * @returns {void}
     */
    expandedChanged() {
        if (this.parent) {
            this.setAttribute('aria-expanded', this.expanded ? 'true' : 'false');
        }
    }
    /**
     * Called after the component is first rendered
     * @param changedProperties Properties which have changed
     * @returns {void}
     */
    firstUpdated(changedProperties) {
        super.firstUpdated(changedProperties);
        this.setAttribute('aria-level', String(this.depth + 1));
    }
    /**
     * Invoked before update() to compute values needed during the update.
     * @param changedProperties changed properties
     * @returns {void}
     */
    willUpdate(changedProperties) {
        if (changedProperties.has('checkedState')) {
            this.checkedChanged();
        }
        if (changedProperties.has('multiple') && changedProperties.get('multiple') !== undefined) {
            this.multipleChanged();
        }
        if (changedProperties.has('expanded')) {
            this.expandedChanged();
        }
    }
    /**
     * A `TemplateResult` that will be used
     * to render the updated internal template.
     * @returns Render template
     */
    render() {
        return html `
      ${this.indentTemplate} ${this.toggleTemplate} ${this.checkboxTemplate} ${this.iconTemplate}
      <div part="label">
        <slot>${this.label}</slot>
      </div>
    `;
    }
};
__decorate([
    property({ attribute: false })
], TreeItem.prototype, "checkedState", void 0);
__decorate([
    property({ type: Boolean, reflect: true, attribute: 'group' })
], TreeItem.prototype, "parent", void 0);
__decorate([
    property({ type: Boolean, reflect: true })
], TreeItem.prototype, "multiple", void 0);
__decorate([
    property({ type: Boolean })
], TreeItem.prototype, "expanded", void 0);
__decorate([
    property({ type: Number, reflect: true })
], TreeItem.prototype, "depth", void 0);
__decorate([
    property({ type: String })
], TreeItem.prototype, "label", void 0);
__decorate([
    property({ type: String })
], TreeItem.prototype, "icon", void 0);
__decorate([
    property({ attribute: false })
], TreeItem.prototype, "item", void 0);
__decorate([
    property({ reflect: true, type: Boolean })
], TreeItem.prototype, "highlighted", void 0);
TreeItem = __decorate([
    customElement('ef-tree-item')
], TreeItem);
export { TreeItem };
