import { __decorate } from "tslib";
import { customElement } from '@refinitiv-ui/core/decorators/custom-element.js';
import { property } from '@refinitiv-ui/core/decorators/property.js';
import { List, valueFormatWarning } from '../../list/index.js';
import { VERSION } from '../../version.js';
import { createDefaultFilter } from '../helpers/filter.js';
import { createTreeRenderer } from '../helpers/renderer.js';
import { TreeManager, TreeManagerMode } from '../managers/tree-manager.js';
import './tree-item.js';
const EXPAND_TOGGLE_ATTR = 'expand-toggle';
/**
 * Displays a tree structure
 * to be used for menus and group selections
 *
 * @fires value-changed - Fired when the user commits a value change. The event is not triggered if `value` property is changed programmatically.
 * @fires expanded-changed - Fired when the user expands or collapses item. The event is not triggered if `expanded` property is changed programmatically.
 *
 * @attr {boolean} [stateless=false] - Disable selections
 * @prop {boolean} [stateless=false] - Disable selections
 */
let Tree = class Tree extends List {
    constructor() {
        super(...arguments);
        this.defaultRole = 'tree';
        this._manager = new TreeManager(this.composer);
        /**
         * Allows multiple items to be selected
         */
        this.multiple = false;
        /**
         * Breaks the relationship when multiple
         * selection mode is enabled
         */
        this.noRelation = false;
        /**
         * Query string applied to tree
         */
        this.query = '';
        /**
         * Custom filter for static data
         * @type {TreeFilter<T> | null}
         */
        this.filter = createDefaultFilter(this);
        /**
         * Renderer used for generating tree items
         */
        this.renderer = createTreeRenderer(this);
    }
    /**
     * Element version number
     * @returns version number
     */
    static get version() {
        return VERSION;
    }
    /**
     * Tree manager used for item manipulation
     */
    get manager() {
        return this._manager;
    }
    /**
     * Expands all groups
     * @returns {void}
     */
    expandAll() {
        this._manager.expandAllItems();
    }
    /**
     * Collapses all groups
     * @returns {void}
     */
    collapseAll() {
        this._manager.collapseAllItems();
    }
    /**
     * Checks all editable items
     * @returns {void}
     */
    checkAll() {
        if (!this.multiple) {
            throw new RangeError('You cannot check all items in single selection mode');
        }
        this._manager.checkAllItems();
    }
    /**
     * Unchecks all editable items
     * @returns {void}
     */
    uncheckAll() {
        this._manager.uncheckAllItems();
    }
    /**
     * @override
     * @ignore
     */
    selectItem(item) {
        // Stateless tree
        if (this.stateless) {
            // Single selection - expand/collapse group (parent)
            if (!this.multiple && this._manager.isItemParent(item)) {
                this.toggleExpandedState(item);
            }
            return false;
        }
        // Multiple selection
        if (this.multiple) {
            return this._manager.toggleItem(item);
        }
        // Single selection - expand/collapse group (parent)
        if (this._manager.isItemParent(item)) {
            this.toggleExpandedState(item);
            return false;
        }
        // Single selection - check item
        if (this._manager.checkItem(item)) {
            this._manager.checkedItems.forEach((checkedItem) => {
                checkedItem !== item && this.forceUncheckItem(checkedItem);
            });
            return true;
        }
        return false;
    }
    /**
     * Force uncheck item when item is locked
     * @param item Original data item
     * @returns {void}
     */
    forceUncheckItem(item) {
        const result = this.composer.unlockItem(item);
        this._manager.uncheckItem(item);
        result && this.composer.lockItem(item);
    }
    /**
     * Dispatches an event, detailing which item has recently changed it's expanded state.
     * @param item Data item of which the expanded property changed
     * @returns {void}
     */
    dispatchExpandedChangedEvent(item) {
        /**
         * Property `detail.value` is the current expanded state.
         * Property `detail.item` is the original data item of which the property has been modified.
         */
        const event = new CustomEvent('expanded-changed', {
            bubbles: false,
            cancelable: false,
            composed: true,
            detail: {
                value: this._manager.isItemExpanded(item),
                item
            }
        });
        this.dispatchEvent(event);
    }
    /**
     * Handles tap event when composed path contains
     * an element with an `expand-toggle` attribute.
     * @param event Tap event to try and handle
     * @returns True or False depending if the event was handled
     */
    handleExpandCollapse(event) {
        const containsToggle = event
            .composedPath()
            .some((target) => target instanceof HTMLElement && target.hasAttribute(EXPAND_TOGGLE_ATTR));
        const itemElement = containsToggle && this.findItemElementFromTarget(event.target);
        const item = itemElement && this.itemFromElement(itemElement);
        if (item) {
            this.toggleExpandedState(item);
            return true;
        }
        return false;
    }
    /**
     * @override
     */
    onTap(event) {
        if (this.handleExpandCollapse(event)) {
            return;
        }
        super.onTap(event);
    }
    /**
     * @override
     */
    onKeyDown(event) {
        switch (event.key) {
            case 'ArrowLeft':
                this.left();
                break;
            case 'ArrowRight':
                this.right();
                break;
            default:
                return super.onKeyDown(event);
        }
        event.preventDefault();
    }
    /**
     * Performs left arrow keyboard action,
     * collapsing a group item if possible.
     * @returns {void}
     */
    left() {
        const item = this.highlightElement && this.itemFromElement(this.highlightElement);
        if (item && this._manager.isItemExpanded(item)) {
            this._manager.collapseItem(item);
            this.dispatchExpandedChangedEvent(item);
        }
    }
    /**
     * Performs right arrow keyboard action,
     * expanding a group item if possible.
     * @returns {void}
     */
    right() {
        const item = this.highlightElement && this.itemFromElement(this.highlightElement);
        if (item && !this._manager.isItemExpanded(item)) {
            this._manager.expandItem(item);
            this.dispatchExpandedChangedEvent(item);
        }
    }
    /**
     * Toggles the expanded state of an item.
     * Executed when a user taps on an item to expand/collapse the group.
     * @param item Original data item
     * @returns {void}
     */
    toggleExpandedState(item) {
        if (this._manager.isItemExpanded(item)) {
            this._manager.collapseItem(item);
        }
        else {
            this._manager.expandItem(item);
        }
        this.dispatchExpandedChangedEvent(item);
    }
    /**
     * @override
     */
    queryItems(engine) {
        return this.composer.queryItems(engine, Infinity);
    }
    /**
     * @override
     */
    queryItemsByPropertyValue(property, value) {
        return this.composer.queryItemsByPropertyValue(property, value, Infinity);
    }
    /**
     * @override
     */
    willUpdate(changeProperties) {
        super.willUpdate(changeProperties);
        if (changeProperties.has('noRelation') || changeProperties.has('multiple')) {
            this._manager.setMode(this.mode);
        }
        if (changeProperties.has('query') || changeProperties.has('data') || changeProperties.has('filter')) {
            this.filterItems();
        }
    }
    /**
     * Filter the internal items by query changes items' hidden state
     * @returns {void}
     */
    filterItems() {
        // if filter is null, it is off and external app is responsible
        if (this.filter) {
            const filter = this.filter;
            const items = this.queryItems((item) => {
                // Do not filter hidden items
                // TODO: blocking `hidden` update via TreeManager/TreeNode of this component
                if (item.hidden) {
                    return false;
                }
                const result = filter(item, this._manager);
                if (result) {
                    this._manager.includeItem(item);
                }
                else {
                    this._manager.excludeItem(item);
                }
                return result;
            }).slice();
            // Do not expand tree if there is no filter applied
            if (this.query) {
                /**
                 * Add all descendants of item list to make the descendants
                 * are accessible which user can navigate into the nested data
                 */
                this.addItemDescendantsToRender(items);
                /**
                 * Add all parents of item list which they must be shown
                 * when some descendant is shown.
                 */
                this.addExpandedAncestorsToRender(items);
            }
        }
    }
    /**
     * Utility method
     * Adds descendants for each item passed
     * @param items List of child items
     * @returns {void}
     */
    addItemDescendantsToRender(items) {
        items.forEach((item) => {
            /**
             * Collapse an item to prevent tree show too many nested expanded
             */
            if (this._manager.isItemExpanded(item)) {
                this._manager.collapseItem(item);
            }
            /**
             * show all descendants of items to make them all are selectable
             * and user can navigate into nested data
             */
            const children = this.composer.getItemChildren(item);
            if (children.length) {
                this.addNestedItemsToRender(children, items, false);
            }
        });
    }
    /**
     * Utility method
     * Add nested children of item list
     * @param items List of items
     * @param excludeItems List of exclude items
     * @param [includeHidden=false] Include hidden items
     * @returns {void}
     */
    addNestedItemsToRender(items, excludeItems, includeHidden = false) {
        items.forEach((item) => {
            // Skip hidden and exclude item
            if (!item.hidden && !excludeItems.includes(item)) {
                // Add item and nested children
                this._manager.includeItem(item);
                const children = this._manager.getItemChildren(item);
                if (children.length) {
                    this.addNestedItemsToRender(children, excludeItems, includeHidden);
                }
            }
        });
    }
    /**
     * Utility method
     * Adds ancestors for each item passed and expand
     * @param items List of child items
     * @returns {void}
     */
    addExpandedAncestorsToRender(items) {
        // Establish unique ancestors set
        const ancestors = new Set();
        // we iterate each item match so as to find ancestors
        items.forEach((item) => {
            // Get the ancestors
            const parent = this._manager.getItemParent(item);
            if (parent && !ancestors.has(parent)) {
                this._manager.getItemAncestors(item).forEach((ancestor) => {
                    ancestors.add(ancestor); // track ancestors
                    this.addExpandedAncestorToRender(ancestor);
                });
            }
        });
    }
    /**
     * Utility method
     * Adds parent and expands
     * @param ancestor parent item
     * @returns {void}
     */
    addExpandedAncestorToRender(ancestor) {
        this._manager.includeItem(ancestor);
        this._manager.expandItem(ancestor);
    }
    /**
     * Selected items in tree
     * @override
     * @type {string[]}
     * @default []
     */
    get values() {
        return this._manager.checkedItems.map((item) => {
            return this.composer.getItemPropertyValue(item, 'value') ?? '';
        });
    }
    set values(values) {
        if (!Array.isArray(values)) {
            valueFormatWarning.show();
            this.values = [];
        }
        else {
            // Clone value arrays
            const newValue = [...values];
            const oldValue = [...this.values];
            if (newValue.toString() !== oldValue.toString()) {
                this._manager.uncheckAllItems();
                values.some((value) => {
                    this.queryItemsByPropertyValue('value', value).forEach((item) => {
                        this._manager.checkItem(item);
                    });
                    return !this.multiple; // Only set the fist value if multiple is not enabled
                });
                this.requestUpdate('values', this.values);
            }
        }
    }
    /**
     * Data object to be used for creating tree
     * @override
     * @type {TreeData<T>}
     */
    get data() {
        return super.data;
    }
    set data(data) {
        super.data = data;
        this._manager = new TreeManager(this.composer, this.mode);
    }
    /**
     * @override
     */
    get renderItems() {
        return this._manager.visibleItems;
    }
    /**
     * Mode to use in the tree manager
     */
    get mode() {
        return !this.multiple || !this.noRelation ? TreeManagerMode.RELATIONAL : TreeManagerMode.INDEPENDENT;
    }
};
__decorate([
    property({ type: Boolean })
], Tree.prototype, "multiple", void 0);
__decorate([
    property({ attribute: 'no-relation', type: Boolean })
], Tree.prototype, "noRelation", void 0);
__decorate([
    property({ type: String })
], Tree.prototype, "query", void 0);
__decorate([
    property({ attribute: false })
], Tree.prototype, "filter", void 0);
__decorate([
    property({ attribute: false })
], Tree.prototype, "renderer", void 0);
Tree = __decorate([
    customElement('ef-tree')
], Tree);
export { Tree };
