import { CollectionComposer } from '@refinitiv-ui/utils/collection.js';
import { TreeNode } from './tree-node.js';
export var CheckedState;
(function (CheckedState) {
    CheckedState[CheckedState["CHECKED"] = 1] = "CHECKED";
    CheckedState[CheckedState["UNCHECKED"] = 0] = "UNCHECKED";
    CheckedState[CheckedState["INDETERMINATE"] = -1] = "INDETERMINATE";
})(CheckedState || (CheckedState = {}));
export var TreeManagerMode;
(function (TreeManagerMode) {
    /**
     * Maintains relationship states across children and parents.
     */
    TreeManagerMode[TreeManagerMode["RELATIONAL"] = 1] = "RELATIONAL";
    /**
     * Items are independent of each other and do not maintain relationship states.
     */
    TreeManagerMode[TreeManagerMode["INDEPENDENT"] = 0] = "INDEPENDENT";
})(TreeManagerMode || (TreeManagerMode = {}));
export class TreeManager {
    /**
     * Collection composer used for managing the data
     */
    get composer() {
        return this._composer;
    }
    /**
     * Most of the time, there is no need to create a new instance of Tree Manager manually.
     * Use the existing instance in components instead.
     * @param input Items or CollectionComposer to be managed.
     * @param mode A mode describing how items are managed either relationally or independently.
     */
    constructor(input, mode = TreeManagerMode.RELATIONAL) {
        /**
         * Mode (algorithm) the tree manage is using
         */
        this.mode = TreeManagerMode.RELATIONAL;
        /** Cache map of TreeNode improving performance */
        this.treeNodeCache = new Map();
        this._composer = input instanceof CollectionComposer ? input : new CollectionComposer(input);
        this.mode = mode;
    }
    /**
     * Returns all items as an array of `TreeNode`.
     * @returns Array of `TreeNode` representing all items
     */
    getTreeNodes() {
        const result = [];
        for (const item of this.items) {
            let treeNode = this.treeNodeCache.get(item);
            if (!treeNode) {
                treeNode = new TreeNode(item, this);
                this.treeNodeCache.set(item, treeNode);
            }
            result.push(treeNode);
        }
        return result;
    }
    /**
     * Returns a `TreeNode` of the original data item.
     * If the item doesn't exist, returns `null`.
     * @param item Original data item
     * @returns `TreeNode` of the original data item or `null`
     */
    getTreeNode(item) {
        let treeNode = this.treeNodeCache.get(item);
        if (!treeNode) {
            const existingItems = this._composer.queryItems((_item) => item === _item, Infinity);
            if (existingItems.length === 0) {
                return null;
            }
            treeNode = new TreeNode(item, this);
            this.treeNodeCache.set(item, treeNode);
        }
        return treeNode;
    }
    /**
     * Is the manager maintaining parent/child relationships
     */
    get manageRelationships() {
        return this.mode === TreeManagerMode.RELATIONAL;
    }
    /**
     * Returns all items in the tree
     */
    get items() {
        return this._composer.queryItems(() => true, Infinity);
    }
    /**
     * Returns all items with children
     */
    get parentItems() {
        return this.items.filter((item) => this.isItemParent(item));
    }
    /**
     * Returns all selected items.
     * When managing relationships, this excludes groups/parents from the result.
     */
    get checkedItems() {
        const items = this._composer.queryItems((item) => {
            if (this.manageRelationships && this.isItemParent(item)) {
                return false;
            }
            return this.isItemChecked(item);
        }, Infinity);
        return Object.freeze(items.slice().sort(this.orderBySelectedAt));
    }
    /**
     * Compare items function order by sequential selected timestamp
     */
    get orderBySelectedAt() {
        // Order by sequential selected timestamp
        return (itemA, itemB) => {
            const timeA = this._composer.getItemPropertyValue(itemA, 'selectedAt') ?? 0;
            const timeB = this._composer.getItemPropertyValue(itemB, 'selectedAt') ?? 0;
            return timeA - timeB;
        };
    }
    /**
     * Returns items which their selected state can be changed.
     * Hidden, disabled or readonly items are not included.
     */
    get editableItems() {
        const topLevel = this._composer.queryItems(() => true, 0);
        return this.getEditableItems(topLevel);
    }
    /**
     * Internal query for getting visible items/nodes
     * @param items Data item collection
     * @param result Resulting array of visible items
     * @returns Collection of visible items
     */
    getEditableItems(items, result = []) {
        for (const item of items) {
            if (this.isItemCheckable(item)) {
                result.push(item);
                const children = this.getItemChildren(item);
                children.length && this.getEditableItems(children, result);
            }
        }
        return result;
    }
    /**
     * Returns currently displayed items.
     * Hidden and children of unexpanded items are not included.
     */
    get visibleItems() {
        const topLevel = this._composer.queryItems(() => true, 0);
        return this.getVisibleItems(topLevel);
    }
    /**
     * Internal query for getting visible items/nodes
     * @param items Data item collection
     * @param result Resulting array of visible items
     * @returns Collection of visible items
     */
    getVisibleItems(items, result = []) {
        for (const item of items) {
            if (!this.isItemHidden(item) && result.push(item) && this.isItemExpanded(item)) {
                const children = this.getItemChildren(item);
                children.length && this.getVisibleItems(children, result);
            }
        }
        return result;
    }
    /**
     * Is the item hidden?
     * @param item Original data item
     * @returns `True` if the item is hidden
     */
    isItemHidden(item) {
        return this._composer.getItemPropertyValue(item, 'hidden') === true;
    }
    /**
     * Is the item checked?
     * @param item Original data item
     * @returns `True` if the item is checked
     */
    isItemChecked(item) {
        if (this.manageRelationships && this.isItemParent(item)) {
            return !this.getItemChildren(item).some((child) => !this.isItemChecked(child));
        }
        return this._composer.getItemPropertyValue(item, 'selected') === true;
    }
    /**
     * Is the item checked indeterminately?
     * @param item Original data item
     * @returns `True` if the item has managed relationships and contains checked descendants
     */
    isItemCheckedIndeterminate(item) {
        if (this.manageRelationships && this.isItemParent(item)) {
            return this.getItemDescendants(item).some((desc) => this.isItemChecked(desc));
        }
        return false;
    }
    /**
     * Determines whether the item is unchecked and can be changed to a checked state.
     * @param item Original data item
     * @returns True if the item can be changed to 'checked'.
     */
    canCheckItem(item) {
        if (this.manageRelationships && this.isItemParent(item)) {
            return this.getItemChildren(item).some((child) => this.canCheckItem(child));
        }
        return this.isItemCheckable(item) && this._composer.getItemPropertyValue(item, 'selected') !== true;
    }
    /**
     * Determines whether the item is checked and can be changed to an unchecked state.
     * @param item Original data item
     * @returns True if the item can be changed to 'unchecked'.
     */
    canUncheckItem(item) {
        if (this.manageRelationships && this.isItemParent(item)) {
            return this.getItemChildren(item).some((child) => this.canUncheckItem(child));
        }
        return this.isItemCheckable(item) && this._composer.getItemPropertyValue(item, 'selected') === true;
    }
    /**
     * Makes an item visible
     * @param item Original data item
     * @returns {void}
     */
    showItem(item) {
        this._composer.setItemPropertyValue(item, 'hidden', false);
        this.updateItem(item); // Make sure the item is updated
    }
    /**
     * Hides an item from the visible collection
     * @param item Original data item
     * @returns {void}
     */
    hideItem(item) {
        this._composer.setItemPropertyValue(item, 'hidden', true);
    }
    /**
     * Forces a modification event, so that the renderer can update.
     * @param item Item of which to find path
     * @returns {void}
     */
    forceUpdateOnPath(item) {
        const path = [...this.getItemAncestors(item), item];
        path.forEach((item) => this._composer.updateItemTimestamp(item));
    }
    /**
     * TODO: find a way to keep `noRelation` of Tree & Tree Select component in-sync
     * Sets the mode (algorithm) the manager should use
     * @hidden Mode updating doesn't sync back up Tree component.
     * @param mode Tree manager mode
     * @returns {void}
     */
    setMode(mode) {
        this.mode = mode;
        // Force update of all visible parent items, making sure any indeterminate states are remove.
        this.parentItems.forEach((item) => this.updateItem(item));
    }
    /**
     * Requests the item to be rerendered manually.
     * Typically, this is not required. The render is triggered automatically when item's properties are updated.
     * @param item Original data item
     * @returns {void}
     */
    updateItem(item) {
        this._composer.updateItemTimestamp(item);
    }
    /**
     * Shows the item.
     * @hidden `hidden` usage in filterItems of Tree & Tree Select component conflicts with this API
     * @param item Original data item
     * @returns `True` if the item is newly included
     */
    includeItem(item) {
        const result = this._composer.unlockItem(item);
        this.showItem(item); // Item must be unlocked first
        return result;
    }
    /**
     * Hides the item.
     * @hidden `hidden` usage in filterItems of Tree & Tree Select component conflicts with this API
     * @param item Original data item
     * @returns `True` if the item is newly excluded
     */
    excludeItem(item) {
        this.hideItem(item);
        return this._composer.lockItem(item);
    }
    /**
     * Returns whether the selected state of item can be changed or not.
     * @param item Original data item
     * @returns `True` if the item is not disabled or readonly
     */
    isItemCheckable(item) {
        return (!this._composer.isItemLocked(item) &&
            this._composer.getItemPropertyValue(item, 'disabled') !== true &&
            this._composer.getItemPropertyValue(item, 'readonly') !== true);
    }
    /**
     * Returns the current expanded state of the item.
     * @param item Original data item
     * @returns `True` if the item is currently expanded so its children are visible.
     */
    isItemExpanded(item) {
        return this.isItemParent(item) && this._composer.getItemPropertyValue(item, 'expanded') === true;
    }
    /**
     * Returns whether the item contains any children or not.
     * @param item Original data item
     * @returns `True` if the item has children
     */
    isItemParent(item) {
        return this._composer.isItemParent(item);
    }
    /**
     * Returns whether the item has a parent or not.
     * @param item Original data item
     * @returns `True` if the item has a parent
     */
    isItemChild(item) {
        return this._composer.isItemChild(item);
    }
    /**
     * Return checked state of the item.
     * @param item Original data item
     * @returns item checked state: CHECKED (1), UNCHECKED (0), INDETERMINATE (-1)
     */
    getItemCheckedState(item) {
        if (this.isItemChecked(item)) {
            return CheckedState.CHECKED;
        }
        if (this.isItemCheckedIndeterminate(item)) {
            return CheckedState.INDETERMINATE;
        }
        return CheckedState.UNCHECKED;
    }
    /**
     * Returns all ancestors of the item.
     * @param item Original data item
     * @returns An array of ancestors
     */
    getItemAncestors(item) {
        return this._composer.getItemAncestors(item);
    }
    /**
     * Returns all descendants of the item.
     * @param item Original data item
     * @param depth Depth of descendants to get. If it's `undefined`, get all descendants.
     * @returns An array of descendants
     */
    getItemDescendants(item, depth) {
        return this._composer.getItemDescendants(item, depth);
    }
    /**
     * Returns the parent of the item, if it has one.
     * @param item Original data item
     * @returns Item parent or `null`
     */
    getItemParent(item) {
        return this._composer.getItemParent(item);
    }
    /**
     * Returns the children of the item as an array.
     * @param item Original data item
     * @returns An array of children
     */
    getItemChildren(item) {
        return this._composer.getItemChildren(item);
    }
    /**
     * Expands the item to show its children.
     * @param item Original data item
     * @returns {void}
     */
    expandItem(item) {
        if (this.isItemParent(item)) {
            this._composer.setItemPropertyValue(item, 'expanded', true);
        }
    }
    /**
     * Collapses the item to hide its children.
     * @param item Original data item
     * @returns {void}
     */
    collapseItem(item) {
        if (this.isItemParent(item)) {
            this._composer.setItemPropertyValue(item, 'expanded', false);
        }
    }
    /**
     * Expands all items.
     * @returns {void}
     */
    expandAllItems() {
        this.parentItems.forEach((item) => this.expandItem(item));
    }
    /**
     * Collapses all items.
     * @returns {void}
     */
    collapseAllItems() {
        this.parentItems.forEach((item) => this.collapseItem(item));
    }
    /**
     * Selects the item.
     * @param item Original data item
     * @returns `True` if the item is modified
     */
    checkItem(item) {
        return this._checkItem(item);
    }
    _checkItem(item, manageRelationships = this.manageRelationships) {
        if (this.canCheckItem(item)) {
            // Create unique timestamp base on the latest selection for sequential selection.
            const timestamp = Date.now();
            this.lastSelectedAt =
                this.lastSelectedAt && this.lastSelectedAt >= timestamp ? this.lastSelectedAt + 1 : timestamp;
            // Set item selected with timestamp
            this._composer.setItemPropertyValue(item, 'selected', true);
            this._composer.setItemPropertyValue(item, 'selectedAt', this.lastSelectedAt);
            if (manageRelationships) {
                this.forceUpdateOnPath(item);
                this.getItemDescendants(item).forEach((descendant) => this._checkItem(descendant, false));
            }
            return true;
        }
        return false;
    }
    /**
     * Deselects the item.
     * @param item Original data item
     * @returns `True` if the item is modified
     */
    uncheckItem(item) {
        return this._uncheckItem(item);
    }
    _uncheckItem(item, manageRelationships = this.manageRelationships) {
        if (this.canUncheckItem(item)) {
            this._composer.setItemPropertyValue(item, 'selected', false);
            if (manageRelationships) {
                this.forceUpdateOnPath(item);
                this.getItemDescendants(item).forEach((descendant) => this._uncheckItem(descendant, false));
            }
            this.updateItem(item);
            return true;
        }
        return false;
    }
    /**
     * Toggle the selected state of the item.
     * @param item Original data item
     * @returns `true` if the item is modified successfully.
     */
    toggleItem(item) {
        return this.checkItem(item) || this.uncheckItem(item);
    }
    /**
     * Selects all items.
     * @returns {void}
     */
    checkAllItems() {
        this.editableItems.forEach((item) => this.checkItem(item));
    }
    /**
     * Deselects all items.
     * @returns {void}
     */
    uncheckAllItems() {
        // uncheck items from top levels when manage relationships to avoid redundant re-renders
        const items = this.manageRelationships ? this._composer.queryItems(() => true, 0) : this.checkedItems;
        items.forEach((item) => this.uncheckItem(item));
    }
}
